/*
 * Copyright (C) 2011 Leonel Iturralde, Andrea Paternesi, Ulf Lorenz, Premkumar Bhaskal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import java.util.Collections;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import org.freelords.army.exception.ArmyIsNotSplittableException;
import org.freelords.army.exception.ArmyTooBigException;
import org.freelords.army.exception.UnitNotInTheArmyException;
import org.freelords.map.PlayerPropertyEntity;
import org.freelords.map.tile.MapTile;

@XmlRootElement(name = "army")
@XmlAccessorType(XmlAccessType.FIELD)
public class Army extends PlayerPropertyEntity {

	/**
	 * List of units contained within the class
	 */
	@XmlElement(name = "unit")
	private List<Unit> units;

	/**
	 * Constant that Holds the maximum number of Units within an Army
	 */
	public static final int MAX_SIZE = 9;

	/**
	 * a default Constructor (needed for JAXB (un)marshalling)
	 */
	@SuppressWarnings("unused")
	private Army() {
	}

	/**
	 * Constructor of Army
	 * 
	 * @param units
	 *            List of Units to be stored within the Army
	 * @throws IllegalArgumentException
	 *             if the unit list is empty
	 * @throws ArmyTooBigException
	 *             if the size of the list <code>units</code> is bigger than
	 *             <code>MAX_SIZE</code>
	 */
	public Army(List<Unit> units, MapTile mapTile)
			throws IllegalArgumentException, ArmyTooBigException {
		if (units.isEmpty()) {
			throw new IllegalArgumentException(
					"Empty army must not be created!");
		}
		if (units.size() > MAX_SIZE) {
			throw new ArmyTooBigException("The army must not be larger than "
					+ MAX_SIZE + " units");
		}
		this.units = units;
		this.setMapTile(mapTile);
	}

	/**
	 * Getter for units field
	 * 
	 * @return List of units stored in the Army
	 */
	public List<Unit> getUnits() {
		return Collections.unmodifiableList(units);
	}

	/**
	 * Determine If the Army can be split
	 * 
	 * @return true if list of units has more than 1 elements and false
	 *         otherwise
	 */
	public boolean isSplittable() {
		return units.size() > 1;
	}

	/**
	 * Determine If the Army can be merged with the other Army
	 * 
	 * @param armyToMerge
	 *            Army that will be merged with this Army
	 * @return true if their units sizes added are equal or less than the
	 *         MAX_SIZE and false if not
	 */
	public boolean isMergeable(Army armyToMerge) {
		return (units.size() + armyToMerge.getUnits().size()) <= MAX_SIZE;
	}

	/**
	 * Merge two Armies
	 * 
	 * @param armyToMerge
	 *            Army that will be absorbed into this army.
	 * @throws ArmyTooBigException
	 *             Thrown if the Merged Army exceeds the limit of MAX_SIZE
	 */
	public void merge(Army armyToMerge) throws ArmyTooBigException {
		if (!isMergeable(armyToMerge)) {
			throw new ArmyTooBigException(
					"The Two Armies merged exceeds the limit of " + MAX_SIZE);
		}
		units.addAll(armyToMerge.getUnits());
	}

	/**
	 * Split Army in two Armies in the same position
	 * 
	 * @param armiesToSplit
	 *            List of Units within The Army that will be split into a new
	 *            army.
	 * @return A new Army with the split-off units at the same map position.
	 * @throws IllegalArgumentException
	 *             Thrown if the list unitsToSplit is empty
	 * @throws ArmyIsNotSplittableException
	 *             Thrown when trying to split an Army that cannot be split
	 *             (size 1)
	 * @throws UnitNotInTheArmyException
	 *             Thrown if the units on the list are not within the stack
	 */
	public Army split(List<Unit> unitsToSplit) throws IllegalArgumentException,
			ArmyTooBigException, ArmyIsNotSplittableException,
			UnitNotInTheArmyException {
		if (!isSplittable()) {
			throw new ArmyIsNotSplittableException(
					"This Army cannot be splitted");
		}
		if (unitsToSplit.isEmpty()) {
			throw new IllegalArgumentException(
					"The Unit List to be splitted cannot be empty");
		}
		if (!units.removeAll(unitsToSplit)) {
			throw new UnitNotInTheArmyException(
					("The Units on the List are not in The Army"));
		}

		return new Army(unitsToSplit, this.getMapTile());
	}

	/**
	 * Get the Movement Points of the Army
	 * 
	 * @return the movement points of the unit within the Army with the least
	 *         Movement Points
	 */
	public Integer getMovementPoints() {
		Integer minimum = null;
		for (Unit unit : units) {
			if (minimum == null) {
				minimum = unit.getMovementPoints();
			}
			if (minimum > unit.getMovementPoints()) {
				minimum = unit.getMovementPoints();
			}
		}
		return minimum;
	}

	/**
	 * Delete Units from the Army. Note: The Army must not be empty after
	 * removal
	 * 
	 * @param unitsToDelete
	 *            A list of Units to delete
	 * @throws IllegalArgumentException
	 *             if the Army is empty after removal
	 * @throws UnitNotInTheArmyException
	 *             if the unitsToDelete are not within the Army or if the
	 *             unitsToDelete is empty
	 */
	public void removeUnits(List<Unit> unitsToDelete)
			throws IllegalArgumentException, UnitNotInTheArmyException {
		if (unitsToDelete.isEmpty() || !units.containsAll(unitsToDelete)) {
			throw new UnitNotInTheArmyException(
					"The list of units to delete must have armies within the Army and must not be Empty");
		}

		if (unitsToDelete.containsAll(units)) {
			throw new IllegalArgumentException(
					"The Army cannot be empty after removal");
		}

		units.removeAll(unitsToDelete);
	}
}
