/*
 * Copyright (C) 2011 Ion Pana, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

import org.freelords.army.jaxb.UnitStatsAdapter;


/** Takes some raw stats and a couple of modifiers, and calculates the modified stats.
 *
 * The class has two purposes. First, it contains the raw stats of an army,
 * which is why it is loadable from xml. Second, you can add modifiers to these
 * stats, in which case this calculator encapsulates all details, and just
 * returns the properly modified stats.
 */

@XmlRootElement(name="statCalculator")
@XmlAccessorType(XmlAccessType.FIELD)
public class StatCalculator {

	@XmlElement(name = "stats")
	@XmlJavaTypeAdapter(UnitStatsAdapter.class)
	private Map<UnitStat, Integer> rawStats;

	@XmlTransient
	private List<Modifier> modifiers;

	public StatCalculator() {
		this.modifiers = new ArrayList<Modifier>();
	}

	/**
	 * Calculates a specific stat based on the raw stats and the present
	 * modifiers.
	 *
	 * Currently, it only works in an additive way - eg: if raw stats have
	 * ATTACK = 1 and there is a Modifier with ATTACK = 4 the result will be 5.
	 * Modifiers can also be negative, though.
	 *
	 * @param statType specifies the stat to be calculated
	 * @return returns the sum of all modifier and raw values for the specified stat type
	 */
	public int calculate(UnitStat statType) {
		int result = rawStats.get(statType);
		for (Modifier modifier : modifiers) {
			result += modifier.getStatValue(statType);
		}
		return result;
	}

	/**
	 * Adds a new modifier.
	 * @param modifier The modifier to add to calculations of the stat.
	 * @throws IllegalArgumentException if the same modifier INSTANCE is already present.
	 */
	public void addModifier(Modifier modifier) {
		//scan for the same instance. Not using contains() because equals() is implemented in Modifier.
		for (Modifier mod : this.modifiers) {
			if (mod == modifier) {
				throw new IllegalArgumentException("The modifier " + modifier + " is already registered.");
			}
		}
		this.modifiers.add(modifier);
	}

	/** Removes an added modifier.
	 * This function silently fails if the modifier has not been added to this calculator.
	 *
	 * @param modifier  the modifier to remove.
	 */
	public void removeModifier(Modifier modifier) {
		this.modifiers.remove(modifier);
	}
}
