/*
 * Copyright (C) 2011 Pana Ion, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/** Elementary building block of an army.
 *
 * As the name suggests, a unit is the fundamental building block of the
 * military, such as a dragon or a squad of pikemen. Every unit in the game is
 * associated to an army, which can be composed of multiple units, though.
 * Note that this class only holds the elementary information that is required
 * to describe the unit and the elementary operations on a unit; all the
 * interaction with other game components is carried out through a manager.
 *
 * @see org.freelords.army.UnitManager
 */
@XmlRootElement(name = "unit")
@XmlAccessorType(XmlAccessType.FIELD)
public class Unit {

	@XmlElement(name = "movementPoints")
	private int movementPoints;
	@XmlElement(name = "id")
	private String id;
	//calculator to be used as a delegate for taking Modifiers into account
	@XmlElement(name = "stat-calculator")
	private StatCalculator calculator;
	@XmlElement(name = "type")
	private String type;

	public Unit() {
	}

	/** Returns the number of movement points left. */
	public int getMovementPoints() {
		return movementPoints;
	}

	/** Sets the number of movement points.
	 *
	 * @param movementPoints The new number of movement points of the unit.
	 */
	public void setMovementPoints(Integer movementPoints) {
		this.movementPoints = movementPoints;
	}

	/** Returns the unique id of the unit. */
	public String getId() {
		return id;
	}

	/** Return the type of the unit.*/
	public String getType() {
		return type;
	}

	/**
	 * Setter for the id field. 
	 *
	 * An id can be set exactly once, then it is permanently attached to a unit.
	 * 
	 * @param newId - the new id.
	 * @throws IllegalStateException if the unit already has an id.
	 */
	public void setId(String newId) {
		if (this.id == null) {
			this.id = newId;
		} else {
			throw new IllegalStateException("This unit already has an id!");
		}
	}

	/**
	 * Sets the calculator to calculate the stats of the unit.
	 * @param calculator the calculator that contains the stats of the unit.
	 */
	public void setCalculator(StatCalculator calculator) {
		this.calculator = calculator;
	}

	/**
	 * Adds a modifier to the calculator instance. 
	 * @see StatCalculator.addModifier()
	 * @param modifier the Modifier to be added
	 */
	public void addModifier(Modifier modifier) {
		this.calculator.addModifier(modifier);
	}

	/**
	 * Removes a modifier from the calculator instance.
	 * @see StatCalculator.removeModifier()
	 * @param modifier The modifier to be removed.
	 */
	public void removeModifier(Modifier modifier) {
		this.calculator.removeModifier(modifier);
	}

	/**
	 * Returns the value of a given UnitStat. Delegates to StatCalculator.
	 * @param statType the unit stat whose value is requested.
	 * @return the value of this stat.
	 */
	public int getUnitStat(UnitStat statType) {
		return this.calculator.calculate(statType);
	}
}
