/*
 * Copyright (C) 2011 Ion Pana, Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import java.io.InputStream;
import org.freelords.army.exception.NoSuchUnitException;
import org.freelords.army.exception.ValidationException;
import java.util.UUID;

import org.freelords.resourcemanager.ResourcePackage;
import org.freelords.resourcemanager.ResourceRepository;
import org.freelords.resourcemanager.ResourceType;
import org.freelords.xmlmanager.XmlManager;

/** Manager used to create new units.
 *
 * This class cares about all details that are required to set up a unit, but
 * which go beyond the horizon of said Unit class. Right now, this involves
 * loading the unit from some repository (i.e., managing the directory
 * information of where the unit's xml file can be found), and doing a few
 * additional checks and setups.
 */
public class UnitManager {

	private XmlManager xmlManager;
	private ResourceRepository repository;

	/**
	 * Constructor for the UnitManager class. 
	 * 
	 * @param xmlManager
	 * 	XmlManager that unmarshalls Unit instances out of xml files. At the moment
	 * 	it is supplied in the constructor and it is expected to be correctly set up
	 * 	- meaning Unit class is registered and instance is initialized.
	 * @param basePath
	 * 	specifies a path to search all the xml files. 
	 */
	public UnitManager(XmlManager xmlManager, ResourceRepository repository) {
		this.xmlManager = xmlManager;
		this.repository = repository;
	}

	/**
	 * Creates an unit instance based on a name. 
	 * Delegates to XmlManager for unmarshalling. 
	 * 
	 * @param unitDefinition the name of the unit type to create.
	 * @return a validated Unit instance
	 * @throws NoSuchUnitException - thrown on every issue that may occur on unmarshalling an unit
	 * @throws ValidationException - thrown if the unmarshalled unit instance is not valid according to game rules
	 */
	public Unit createUnit(String unitDefinition) throws NoSuchUnitException, ValidationException {
		Unit unit = null;

		//retrieve the unit out of the definition file and wrap exceptions in a NoSuchUnitException.
		try {
			InputStream stream = repository.getStream(ResourcePackage.ARMY, unitDefinition, ResourceType.XML);
			unit = this.xmlManager.unmarshall(stream);			

		} catch (Exception e) {
			throw new NoSuchUnitException("Could not create unit with the definition " + unitDefinition, e);
		}

		//validate
		validateUnit(unit);
		//post-creation
		postCreate(unit);

		return unit;
	}

	/**
	 * Validates a unit for adhering to game rules.
	 * 
	 * Right now, these rules are:
	 *
	 * <ul>
	 * 	<li>Movement Points >= 0</li>
	 * </ul>
	 * 
	 * @param unit the unit to be validated
	 * @throws ValidationException throws if one of the validation rules does not apply for the unit in question
	 */
	private void validateUnit(Unit unit) throws ValidationException {
		if (unit.getMovementPoints() < 0) {
			throw new ValidationException("Unit's Movement points must be at least 0.");
		}
	}

	/**
	 * Does additional initialization after loading a Unit from JAXB.
	 * At the moment it sets the unit id to a random UUID.
	 * @param unit a created unit.
	 */
	private void postCreate(Unit unit) {
		unit.setId(UUID.randomUUID().toString());
	}
}
