/*
 * Copyright (C) 2011 Paul Statham
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

package org.freelords.army.heroes;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/** The backpack of a hero.
 *
 * A hero can store an arbitrary amount of items in his backpack. They are just
 * lying around, though. You need to equip items in order to use their powers.
 */

@XmlRootElement(name = "backpack")
public class Backpack {
	
	@XmlElement(name = "item")
	private List<Item> items = new ArrayList<Item>();

	/**
	 * Adds an item to the backpack
	 * 
	 * @param item to store in backpack
	 */
	public void storeItem(Item item) {
		items.add(item);
	}

	/**
	 * Remove the specified item from the backpack
	 * 
	 * @param item the item to remove from list
	 * @throws IllegalArgumentException if the item is not in the backpack.
	 */
	public void removeItem(Item item) {
		if(!items.contains(item)) {
			throw new IllegalArgumentException("This item is not in the backpack");
		}
		items.remove(item);
	}

	/**
	 * Equip item in backpack
	 * 
	 * @param equipment  the equipment that is filled with the item
	 * @param item the item that is equipped
	 * @throws IllegalArgumentException if the equipment is null or if the slot is already filled.
	 */
	public void equipItem(Equipment equipment, Item item) {
		if((equipment != null) && items.contains(item)) {
			equipment.equip(item);
		} else {
			throw new IllegalArgumentException("There was an error equipping this item");
		}		
	}
	
	/**
	 * Remove item from equipment and add it to the backpack.
	 * 
	 * @param equipment equipment from which the item is removed.
	 * @param itemType the type of the item that is removed.
	 * @throws IllegalArgumentException if no item of the given type is equipped.
	 */
	public void takeOffItem(Equipment equipment, ItemType itemType) {
		Item removedItem = equipment.remove(itemType);
		if(removedItem == null) {
			throw new IllegalArgumentException("Item can't be removed from empty slot");
		}
	}

}
