/*
 * Copyright (C) 2011 Ion Pana, Leonel Iturralde
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

package org.freelords.log;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** Central place for logging stuff.
 *
 * The enum has two instances, one for logging on the client side, and one for
 * logging on the server side. It essentially forwards everything to slf4j, using
 * the java logging backend.
 *
 * We use four log levels here:
 * <ul>
 * <li><em>DEBUG</em> are all log messages that are necessary to trace the
 *     detailed execution path. That is, which services were executed, what interesting
 *     stuff was done, which non-critical exceptions were thrown etc.</li>
 * <li><em>INFO</em> denotes log messages that are interesting for normal
 *     players to follow the execution. For example, when the client has
 *     connected or a new game started, you would use this log level.</li>
 * <li><em>WARN</em> denotes errors that have the potential to crash something.
 *     For example, the server would log a warning if a client sends invalid
 *     commands.</li>
 * <li><em>ERROR</em> is reserved for when the game or parts of it are broken,
 *     for example, if you cannot connect to the server, or if the audio system
 *     does not work.</li>
 * </ul>
 */

public enum Log {

	SERVER("[Server] ", "org.freelords.server"),
	CLIENT("[Client] ", "org.freelords.client");

	private static final String LOG_PROPERTIES_FILE = "java.util.logging.config.file";

	private Logger logger;
	private String prefix;

	private Log(String prefix, String logName) {
		this.prefix = prefix;
		if (System.getProperty(LOG_PROPERTIES_FILE) == null) {
			//set a default value
			System.setProperty(LOG_PROPERTIES_FILE, "src/log.properties");
		}
		
		this.logger = LoggerFactory.getLogger(logName);
	}

	public void debug(String message) {
		this.logger.debug(this.prefix + message);
	}

	public void debug(String message, Object object) {
		this.logger.debug(this.prefix + message + object);
	}

	public void debug(String message, Throwable throwable) {
		this.logger.debug(this.prefix + message, throwable);
	}


	public void info(String message) {
		this.logger.info(this.prefix + message);
	}

	public void info(String message, Object object) {
		this.logger.info(this.prefix + message + object);
	}

	public void info(String message, Throwable throwable) {
		this.logger.info(this.prefix + message, throwable);
	}


	public void warn(String message) {
		this.logger.warn(this.prefix + message);
	}

	public void warn(String message, Object object) {
		this.logger.warn(this.prefix + message + object);
	}

	public void warn(String message, Throwable throwable) {
		this.logger.warn(this.prefix + message, throwable);
	}


	public void error(String message) {
		this.logger.error(this.prefix + message);
	}

	public void error(String message, Object object) {
		this.logger.error(this.prefix + message + object);
	}

	public void error(String message, Throwable throwable) {
		this.logger.error(this.prefix + message, throwable);
	}
}
