/*
 * Copyright (C) 2011 Ulf Lorenz
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.main;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.freelords.network.client.FLNetworkClient;
import org.freelords.network.client.Session;
import org.freelords.network.command.ChatBroadcastCommand;
import org.freelords.network.command.ChatMulticastCommand;

/** Reads chat messages from an input stream and sends them to the server.
 *
 * This class reads lines from an input stream, and in general sends them
 * as chat commands to a server. It also interpretes a handful of special
 * character sequences.
 *
 * Special character sequences are:
 * <ul>
 *    <li>"\&lt;number&gt; " anywhere in the text sends the message only to the
 *    recipient(s) with the respective index number.</li>
 *    <li>"\list" discards the message, and instead lists all registered clients
 *    together with an index number, which is used for the addressing.</li>
 *    <li>"\quit" stops the reading.</li>
 *    <li>"\help" prints some help text.</li>
 * </ul>
 *
 */

public class CommandReader {

	private BufferedReader input;
	private PrintStream output;
	private FLNetworkClient client;

	/** Constructs a new command reader.
	 *
	 * @param input   the stream (usually stdin) from which we read the input.
	 * @param client  the network client that we use to send commands.
	 */
	 public CommandReader(InputStream input, PrintStream output, FLNetworkClient client) {
		 this.input  = new BufferedReader(new InputStreamReader(input));
		 this.output = output;
		 this.client = client;
	 }

	 /** Reads from the input until it encounters a "\quit" command.
	  *
	  * Reading happens in a blocking manner.
	  */
	 public void readCommands() throws IOException {
		output.println("Type \"\\help\" for a brief help");

		while (true) {
			String line = input.readLine();

			// empty lines are ignored
			if (line.isEmpty()) {
				continue;
			}

			if (line.contains("\\help")) {
				printHelp();
				continue;
			}

			// lines with a \quit command lead to an immediate stop.
			if (line.contains("\\quit")) {
				client.stop();
				break;
			}

			// lines with a list command list all registered clients
			if (line.contains("\\list")) {
				listClients();
				continue;
			}

			// otherwise, extract and replace the recipients, replace
			// backslashes, and send the chat message.
			Set<String> recipients = extractRecipients(line);
			line = line.replaceAll("\\\\\\d+", "");
			sendMessage(recipients, line);
		}
	 }

	 private void printHelp() {
		 output.println("Type text and hit return to send it.");
		 output.println("Special commands are interpreted if they appear anywhere in the input text:");
		 output.println("\"\\help\"       this help text");
		 output.println("\"\\quit\"       quits the client");
		 output.println("\"\\list\"       lists all client with names and index number");
		 output.println("\"\\<number>\"   sends the message only to the client with this index number.");
		 output.println("                 multiple targets are possible.");
	 }

	 /** Prints the currently registered clients. */
	 private void listClients() {
		Session session = client.getSession();

		if (session.isListClientsIdEmpty()) {
			output.println("No registered clients are known !?!?");
			return;
		}

		int index = 0;
		for (UUID clientId : session.listClientIds()) {
			String line = "[" + index + "]: " + session.getClientName(clientId) 
					+ "  {id = " + clientId + "}";
			if (clientId.equals(session.getCurrentClientId())) {
				line += "   (local)";
			}

			output.println(line);
			index++;
		}
	 }

	 /** Finds all occurences of "\&lt;number&gt;" and looks up the respective client id's.
	  *
	  * @param line   the line to read.
	  * @return a list of the id's of all adressed clients or an empty list if
	  * the message is meant to be broadcast (i.e., if no adressee is found).
	  */
	 private Set<String> extractRecipients(String line) {
		Set<String> recipients = new HashSet<String>();
		
		// The pattern is actually "\\[0-9]+ ", i.e., a backslash followed
		// by one or more digits and a space
		Matcher matcher = Pattern.compile("\\\\\\d+").matcher(line);
		while (matcher.find()) {
			String recipient = matcher.group();
			recipient = recipient.replace("\\", "");

			int recipientIndex = Integer.decode(recipient);
			if (recipientIndex >= client.getSession().listClientIds().size() || recipientIndex < 0) {
				output.println("Error: invalid recipient; number out of bounds.");
			}

			for (UUID recipientId : client.getSession().listClientIds()) {
				if (recipientIndex == 0) {
					recipients.add(recipientId.toString());
					break;
				}
				recipientIndex--;
			}
		}

		return recipients;
	 }

	 /** Sends the chat message to the given recipients.
	  *
	  * @param recipients   the client id's to receive this message, everyone if this is empty.
	  * @param message      the chat message to send.
	  */
	 private void sendMessage(Set<String> recipients, String message) {
		 if (recipients.isEmpty()) {
			 client.sendCommand(new ChatBroadcastCommand(message));
		 }
		 else {
			 client.sendCommand(new ChatMulticastCommand(message, recipients));
		 }
	 }
}
