/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.client;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Client;
import com.esotericsoftware.kryonet.Listener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.freelords.log.Log;
import org.freelords.network.command.FLNetworkCommand;
import org.freelords.network.common.FLNetworkNode;

public class FLNetworkClient implements FLNetworkNode {

	protected List<Class> registeredClasses = new ArrayList<Class>();
	private final Client kryonetClient;
	private boolean isRunning = false;
	private boolean objectsRegistered = false;
	private boolean listenerBound = false;
	private Session session;
	
	public FLNetworkClient(Client kryonetClient, Session session) {
		if (kryonetClient == null) {
			throw new IllegalArgumentException("Must supply a kryonet client.");
		}
		this.kryonetClient = kryonetClient;

		if (session == null) {
			throw new IllegalArgumentException("Must supply a session.");
		}
		this.session = session;
	}

	public Session getSession() {
		return session;
	}

	@Override
	public void registerClass(Class networkClass) {
		registeredClasses.add(networkClass);
	}

	public void addReceivingListener(Listener receiveListener) {
		kryonetClient.addListener(receiveListener);
		this.listenerBound = true;
	}

	public void initNetwork() throws IOException {
		if (registeredClasses.isEmpty()) {
			throw new IOException();
		}

		Kryo kryo = kryonetClient.getKryo();
		for (Class networkClass : registeredClasses) {
			kryo.register(networkClass);
		}

		this.objectsRegistered = true;
		Log.CLIENT.info("Client network initialized.");
	}

	public void start() throws IOException {
		if (isConfigured() && !this.isRunning) {
			kryonetClient.start();
			this.isRunning = true;
			Log.CLIENT.info("Client started.");
		} else {
			throw new IOException();
		}
	}

	public void connect(String serverAddress, Integer serverPort) throws IOException {
		if (isConfigured() && this.isRunning) {
			kryonetClient.connect(5000, serverAddress, serverPort);
			Log.CLIENT.info("Client connected to " + serverAddress + " on port " + serverPort + ".");
		} else {
			throw new IOException();
		}
	}

	public void stop() throws IOException {
		if (isRunning) {
			kryonetClient.stop();
			Log.CLIENT.info("Client stopped.");
		} else {
			throw new IOException();
		}
	}

	public void sendCommand(FLNetworkCommand command) {
		kryonetClient.sendTCP(command);
	}

	private boolean isConfigured() {
		return objectsRegistered && listenerBound;
	}
}
