/*
 * Copyright (C) 2011 Andrea Paternesi, Anton Kirillov, Ulf Lorenz, Paul Statham,Leonel Iturralde
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.client;

import java.util.HashSet;
import java.util.Set;

import org.freelords.log.Log;
import org.freelords.network.client.service.ClientService;
import org.freelords.network.update.Update;
import org.freelords.network.update.UpdateError;

import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Listener;

/**
 * Listens for incoming updates and executes them.
 *
 * This class essentially mirrors the server listener. Arbitrary services can
 * attach to this listener; when a new update comes in, the listener passes
 * these updates around each service.
 */

public class FLNetworkClientListener extends Listener {

	//private static final Logger log = Logger.getLogger(FLNetworkClientListener.class.getName());
	private Set<ClientService> services = new HashSet<ClientService>();

	/**
	 * Registers a service with the listener.
	 */
	public void registerService(ClientService service) {
		this.services.add(service);
	}

	/**
	 * Unregisters a service with the listener.
	 */
	public void unregisterService(ClientService service) {
		this.services.remove(service);
	}

	/**
	 * Called when an object has been received from the remote end of the connection.
	 * @param object the object that was sent over the network. Should implement Update interface
	 */
	@Override
	public void received(Connection connection, Object object) {
		if (!(object instanceof Update)) {
			Log.CLIENT.debug("Received package that does not implement Update interface.",object);
			return;
		}
		if(object instanceof UpdateError) {
			Log.CLIENT.error("Error received.", ((UpdateError)object).getException().getMessage());
			Log.CLIENT.error("Error received.", object);
		}

		Log.CLIENT.debug("Received update: ", object);
		for (ClientService service : this.services) {
			validateAndExecute(service, (Update) object);
		}
	}

	private void validateAndExecute(ClientService service, Update update) {
		try {
			if (service.accept(update)) {
				Log.CLIENT.debug("Executing service: ", service);
				service.execute(update);
			}
		} catch(Exception e) {
			Log.CLIENT.error("Exception during execution of update command", e);
		}
	}
}
