/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.command;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

/** Send a chat message over the Network in a Multicast mode.
 *
 * <p>
 * By setting up and sending an instance of this command, the client can send
 * to all other clients a chat message. When the server receives this command
 * it will send the message to all clients whose id is in the client id list of the command.
 * </p>
 */
public class ChatMulticastCommand implements FLNetworkCommand {

	private String chatText;
	private Set<String> clientIds;

	/** No-args constructor required by kryonet. */
	private ChatMulticastCommand() {
	}

	/** Constructs the command with a given chat text and a list of clients' ids. */
	public ChatMulticastCommand(String chatText, Set<String> clientIds) throws IllegalArgumentException {
		if (chatText == null) {
			throw new IllegalArgumentException("The Chat Message cannot be null");
		}
		if (clientIds == null || clientIds.isEmpty()) {
			throw new IllegalArgumentException("The Multicast id list cannot be null");
		}

		checkForNullIds(clientIds);

		this.chatText = chatText;
		this.clientIds = clientIds;
	}

	/** Returns the text of the chat. */
	public String getChatText() {
		return chatText;
	}

	/** Returns the list of the clients' ids */
	public Set<UUID> getClientIds() {
		Set<UUID> tmp = new HashSet<UUID>();

		for (String uuid : clientIds) {
			if (uuid != null) {
				tmp.add(UUID.fromString(uuid));
			}
		}

		return Collections.unmodifiableSet(tmp);
	}

	/**
	 * Checks if in the list there is a null client id
	 * 
	 * @param clientIds The list of clients' id
	 * @throws IllegalArgumentException 
	 */
	protected void checkForNullIds(Set<String> clientIds) throws IllegalArgumentException {
		for (String clientId : clientIds) {
			if (clientId == null) {
				throw new IllegalArgumentException("The Chat Message cannot be null");
			}
		}
	}
}
