/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Listener;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.freelords.network.common.FLNetworkNode;

/** Cares about the setup and running of the server part.
 *
 * This class sets up and runs an instance of
 * {@link org.freelords.network.server.FLBaseServer FLBaseServer}, which does
 * the low-level IO network stuff. To set up and start a server, you need to
 *
 * <ol>
 * <li>register all objects with kryonet that are sent over the network (children
 *      of {@link org.freelords.network.command.FLNetworkCommand FLNetworkCommand} and
 *      {@link org.freelords.network.update.Update}).</li>
 * <li>bind to a specific TCP port on the server machine.</li>
 * <li>attach a listener that deals with incoming packages (usually an instance of
 *      {@link org.freelords.network.server.FLNetworkServerListener FLNetworkServerListener}).</li>
 * <li>initialize the network.</li>
 * <li>start the server.</li>
 * </ol>
 */

public class FLNetworkServer implements FLNetworkNode {

	protected List<Class> registeredClasses = new ArrayList<Class>();
	private final FLBaseServer kryonetServer;
	private boolean objectsRegistered = false;
	private boolean portIsBound = false;
	private boolean listenerBound = false;
	private boolean isRunning = false;

	/** Creates a new server class.
	 *
	 * @param kryonetServer  an instance for handling the low-level network stuff
	 * @throws IllegalArgumentException if no server is supplied.
	 */
	public FLNetworkServer(FLBaseServer kryonetServer) throws IllegalArgumentException {
		if (kryonetServer == null) {
			throw new IllegalArgumentException("kryonet server was null");
		}
		this.kryonetServer = kryonetServer;
	}

	/** Registers a new class to be sent over the network.
	 *
	 * @param networkCommandClass 
	 *        a class object that specifies a class that can be sent over the
	 *        network.
	 */
	@Override
	public void registerClass(Class networkClass) {
		registeredClasses.add(networkClass);
	}

	/** Binds the server to a specific TCP port.
	 *
	 * @param port  the port to bind to.
	 * @throws IOException if the server cannot bind to the specified port.
	 */
	public void bindPort(int port) throws IOException {
		kryonetServer.bind(port);
		this.portIsBound = true;
	}

	/** Initializes the underlying kryonet server instance.
	 *
	 * @throws IOException if no commands have been registered as to be sent
	 * over the network.
	 */
	public void initNetwork() throws IOException {
		if(registeredClasses.isEmpty()) {
			throw new IOException();
		}
		
		Kryo kryo = kryonetServer.getKryo();
		for (Class networkClass : registeredClasses) {
			kryo.register(networkClass);
		}
		this.objectsRegistered = true;
	}

	/** Adds a listener to handle incoming commands.
	 *
	 * @param receiveListener  a listener that can handle incoming commands.
	 */
	public void addReceivingListener(Listener receiveListener) {
		kryonetServer.addListener(receiveListener);
		this.listenerBound = true;
	}

	/** Starts the server and listens for incoming packages.
	 *
	 * @throws IOException if the server has not been properly configured or if
	 * the server is already running.
	 */
	public void start() throws IOException {
		if (isConfigured() && !this.isRunning) {
			kryonetServer.start();
			this.isRunning = true;
		} else {
			throw new IOException();
		}
	}

	/** Stops the server.
	 *
	 * @throws IOException if the server is not running.
	 */
	public void stop() throws IOException {
		if (isRunning) {
			kryonetServer.stop();
		} else {
			throw new IOException();
		}
	}

	/** Checks whether all configuration steps have been done. */
	private boolean isConfigured() {
		return portIsBound && objectsRegistered && listenerBound;
	}
}
