/*
 * Copyright (C) 2011 Andrea Paternesi, Ion Pana, Ulf Lorenz, Paul Statham, Leonel Iturralde
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server;

import java.util.HashSet;
import java.util.Set;

import org.freelords.log.Log;
import org.freelords.network.server.exception.ServiceException;
import org.freelords.network.command.FLNetworkCommand;
import org.freelords.network.common.FLNetworkConnection;
import org.freelords.network.server.security.FLSecurityManager;
import org.freelords.network.server.service.Service;
import org.freelords.network.update.UpdateError;

import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Listener;

/** Listens for incoming commands and executes them.
 *
 * Kryonet uses the concept of a listener that can be attached to a server, and
 * which is given all objects that the server receives. Our implementation of a
 * listener expects to receive instances of classes that implement the
 * {@link org.freelords.network.command.FLNetworkCommand FLNetworkCommand}
 * interface. It passes this command first to a security executor, which checks
 * if the request is generally valid (e.g., if the client is allowed to perform
 * a certain command). If the command passes this check, it is forwarded to one
 * or more services, which handle the request.
 */

public class FLNetworkServerListener extends Listener {

	private final Updater updater;
	private final FLSecurityManager securityManager;
	private Set<Service> services = new HashSet<Service>();

	/** Creates a new listener.
	 *
	 * @param securityManager  an object that checks the validity of a command request.
	 * @param updater  an object that bundles the means to talk back to the client(s)
	 * @param networkCommandExecutor  an object that executes commands.
	 */
	public FLNetworkServerListener(FLSecurityManager securityManager, Updater updater) {
		this.securityManager = securityManager;
		this.updater = updater;
	}

	/** Callback that is run when the server receives an object.
	 *
	 * @param connection    the connection over which the object was sent.
	 * @param object        the object that was sent over the network.
	 */
	@Override
	public void received(Connection connection, Object object) {
		if (!(object instanceof FLNetworkCommand)) {
			//Kryonet sends additional messages on the wire that are not user defined - eg. KeepAlive messages.
			Log.SERVER.debug("Received unknown message: ", object);
			return;
		}

		FLNetworkConnection nconnection = (FLNetworkConnection) connection;
		FLNetworkCommand command = (FLNetworkCommand) object;

		if (securityManager.checkRights(nconnection, command)) {
			Log.SERVER.debug("Received command: ", command);
			for (Service service : this.services) {
				validateAndExecute(service, command, nconnection);
			}
		} else {
			Log.SERVER.warn("Connection was denied");
			return;
		}
	}

	/**
	 * Registers a service with the listener.
	 * This fails silently if an existing service is registered again.
	 *
	 * @param service  the service that is registered to receive and handle commands.
	 */
	public void registerService(Service service) {
		this.services.add(service);
	}

	/** Unregisters a service with the listener. */
	public void unregisterService(Service service) {
		this.services.remove(service);
	}

	/**
	 * Wrap the service invoking the command with a try/catch all block - if one service fails with an exception,
	 * other services should be able to process the command.
	 *
	 * @param service  the service that is given the command
	 * @param command  the command that was received
	 */
	private void validateAndExecute(Service service, FLNetworkCommand command, FLNetworkConnection connection) {
		try {
			if (service.accept(command)) {
				Log.SERVER.debug("Executing service: ", service);
				service.execute(command, connection, this.updater);
			}
		} catch(Exception e) {
			Log.SERVER.error("Service " + service + " failed with error.", e);

			if(updater != null) {
				updater.sendUpdate(connection.getClientId(), new UpdateError(new ServiceException("Service Failed")));
			}
		}
	}
}
