/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz, Leonel Iturralde
 *               2012 Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server.service;

import java.util.List;
import java.util.Set;
import java.util.UUID;
import org.freelords.log.Log;
import org.freelords.network.command.ChatBroadcastCommand;
import org.freelords.network.command.ChatMulticastCommand;
import org.freelords.network.command.FLNetworkCommand;
import org.freelords.network.command.exception.InvalidCommandException;
import org.freelords.network.common.FLNetworkConnection;
import org.freelords.network.server.Updater;
import org.freelords.network.update.ChatUpdate;

/** Service that takes care about the chat messages.
 *
 * This service processes only command of class ChatBroadcastCommand and
 * ChatMulticastCommand, and sets the id,
 * name, and a preliminary player type for the network connection. After this
 * service has acted, the connection is considered established. If the
 * connection has already been established, it throws an exception instead.
 */
public class ServerChatService implements Service {

	/** This class accepts only chat commands. */
	@Override
	public boolean accept(FLNetworkCommand command) {
		return (command instanceof ChatBroadcastCommand || command instanceof ChatMulticastCommand);
	}

	/** Sends chat messages to clients.
	 *
	 * @param command the chat command that contains the chat message.
	 * @param connection the connection from which is coming the chat message
	 * @param updater object for sending the chat messages back to the world.
	 */
	@Override
	public void execute(FLNetworkCommand command, FLNetworkConnection connection, Updater updater) throws Exception {
		if (command instanceof ChatBroadcastCommand) {
			ChatBroadcastCommand chatBroadcastCommand = (ChatBroadcastCommand) command;
			ChatUpdate chatUpdate = new ChatUpdate(chatBroadcastCommand.getChatText(), connection.getClientId());
			updater.broadcastUpdate(chatUpdate);
			
			Log.SERVER.debug("Recieved a chat broadcastcast command. "+chatUpdate.getMessage());
		}
		if (command instanceof ChatMulticastCommand) {
						
			UUID senderId = connection.getClientId();
						
			ChatMulticastCommand chatMulticastCommand = (ChatMulticastCommand) command;
			Set<UUID> multicastIdList = chatMulticastCommand.getClientIds();

			checkClientIdsList(senderId, updater.listConnections(), multicastIdList);
			ChatUpdate chatUpdate = new ChatUpdate(chatMulticastCommand.getChatText(), senderId);
			sendMulticastUpdate(updater, multicastIdList, chatUpdate);
			
			Log.SERVER.debug("Recieved a chat multicast command from "+connection.getClientName());			
		}
	}

	/**
	 * Used to check if something is wrong with passed client ids.
	 * if the client id does not match with one of the ids of the registered connections or
	 * is null an InvalidCommandException is thrown.
	 *
	 * @param senderId The id of the sender client
	 * @param connectionsList the list of the registered connections
	 * @param clientIds the ids of the clients listed in the multicast command
	 * @throws InvalidCommandException thrown if some client id is null or invalid
	 */
	private void checkClientIdsList(UUID senderId, List<FLNetworkConnection> connectionsList, Set<UUID> clientIds)
			throws InvalidCommandException {

		// check that all ids are not null
		for (UUID clientId : clientIds) {
			if (clientId == null) {
				Log.SERVER.debug("Recieved a client Id null in multicast.");
				throw new InvalidCommandException("A client Id in the multicast list was null.");
			}
		}

		// check that all ids are valid
		for (UUID clientId : clientIds) {
			boolean found = false;
			for (FLNetworkConnection connection : connectionsList) {
				if (clientId.equals(connection.getClientId())) {
					found = true;
					break;
				}
			}

			if (!found) {
				Log.SERVER.debug("A client Id in the multicast list was invalid or not registered on the server."
						+ " (offending id: " + clientId + ")");
				throw new InvalidCommandException("A client Id in the multicast list was invalid or not registered into the server.");
			}
		}
	}

	/**
	 *  Used to send the chat message to the right clients.
	 *
	 * @param updater The updater who sends physically the message ofer the network.
	 * @param multicastIdList The list of the client connections ids to send the message at.
	 * @param chatUpdate The update command to be sent back.
	 */
	private void sendMulticastUpdate(Updater updater, Set<UUID> multicastIdList, ChatUpdate chatUpdate) {
		for (UUID client : multicastIdList) {
			updater.sendUpdate(client, chatUpdate);
		}
	}
}
