/*
 * Copyright (C) 2011 Ion Pana, Ulf Lorenz, Anton Kirillov, Leonel Iturralde
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server.service;

import java.util.List;
import java.util.UUID;

import org.freelords.log.Log;
import org.freelords.network.command.ClientNetworkRegisterCommand;
import org.freelords.network.command.FLNetworkCommand;
import org.freelords.network.common.ClientRole;
import org.freelords.network.common.FLNetworkConnection;
import org.freelords.network.server.Updater;
import org.freelords.network.update.ClientListUpdate;
import org.freelords.network.update.ClientRegistrationUpdate;

/** Service that initializes a network connection.
 *
 * This service processes only ClientNetworkRegisterCommands, and sets the id,
 * name, and a preliminary player type for the network connection. After this
 * service has acted, the connection is considered established. If the
 * connection has already been established, it throws an exception instead.
 */

public class ServerClientRegisterService implements Service {

	/** This class accepts only registration commands. */
	@Override
	public boolean accept(FLNetworkCommand command) {
		return (command instanceof ClientNetworkRegisterCommand);
	}

	/** Initializes the connection for some client.
	 *
	 * @param command  the registration command that contains the requested name for this client.
	 * @param connection  the connection that is to be initialized
	 * @param updater  object for sending the happy news back to the world.
	 *
	 * @throws IllegalStateException if the connection has already been initialized.
	 */
	@Override
	public void execute(FLNetworkCommand command, FLNetworkConnection connection, Updater updater) throws Exception {
		ClientNetworkRegisterCommand registerCommand = (ClientNetworkRegisterCommand) command;

		if (connection.getClientId() == null) {
			UUID clientId = UUID.randomUUID();
			connection.setClientId(clientId);
			connection.setRole(ClientRole.GUEST);

			//set the player name
			if (registerCommand.getClientName() == null) {
				connection.setClientName("new player");
			} else {
				connection.setClientName(registerCommand.getClientName());
			}
			Log.SERVER.info("New Client Connected: " + registerCommand.getClientName() + " with id " + clientId);
			responseNewClient(updater, clientId);
			responseAll(updater);

		}
		//the connection is already initialized, an exception must be thrown
		else {
			//TODO maybe this should be a security exception
			Log.SERVER.warn("Client Connection already initialized. " + connection.getClientName());
			throw new IllegalStateException("Connection is already initialized. " +
					"A second initialization attempt is invalid.");
		}
	}



	/**
	 * Notify new client about successful connection. Send him his UUID
	 */
	private void responseNewClient(Updater updater, UUID clientId) {
		ClientRegistrationUpdate clientUpdate = new ClientRegistrationUpdate(clientId);
		updater.sendUpdate(clientId, clientUpdate);
	}

	/**
	 * Notify all connected clients about new player arrival
	 */
	private void responseAll(Updater updater) {
		ClientListUpdate broadCastUpdate = new ClientListUpdate();
		List<FLNetworkConnection> connections = updater.listConnections();
		for (FLNetworkConnection c : connections) {
			broadCastUpdate.addClient(c.getClientId(), c.getClientName());
		}
		updater.broadcastUpdate(broadCastUpdate);
	}
}
