/*
 * Copyright (C) 2011 Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.resourcemanager;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.JarFileSystem;
import org.openide.filesystems.LocalFileSystem;
import org.openide.filesystems.MultiFileSystem;

/** The standard repository.
 *
 * <p>
 * This class can be supplied any path or jar/zip file (a
 * <em>repository</em>), and it will use the files/directories/file entries
 * below the root to look for requested resources. It relies on
 * org.openide.filesystems for the actual lookup, and just encapsulates the
 * gory details. If you supply multiple repositories, the adjoint of them can
 * be accessed. In case of conflict, the retrieved file depends on the order
 * of the repositories given in the constructor.
 * </p>
 *
 */
public class DataRepository implements ResourceRepository {

	/** A list of files/directories to open as root. */
	private List<String> filelist;
	/** The master repository. */
	private FileSystem repository;
	
	/** Netbeans documentation mentions that File.separator should not be used in 
	 * paths, "/" should always be used.
	 */
	private static final String PATH_SEPARATOR = "/";
	
	/** Creates a new repository.
	 *
	 * Note that the order of the file names matters. In case of conflict,
	 * resources from repositories in the front of the list precede those from
	 * the back of the list.
	 *
	 * @param files
	 *         a list of directories or jar/zip files that serve as roots for
	 *         the newly created file systems.
	 */
	public DataRepository(List<String> files) {
		this.filelist = files;
	}

	/** {@inheritDoc} */
	@Override
	public void connectToResources() throws InvalidFileException {
		List<FileSystem> repList = new ArrayList<FileSystem>();

		// set up all repositories and store them in a list
		for (String file : filelist) {
			File f = new File(file);

			if (f.isDirectory()) {
				openAsFileSystem(f, repList);
			} else if (file.matches(".*\\.zip$") || file.matches(".*\\.jar$")) {
				openAsZipJarFileFileSystem(f, repList);
			} else {
				throw new InvalidFileException(file + "is an invalid resource. Must be either a directory or a jar/zip file.");
			}
		}

		//we always return a multiFileSystem even if we have only one
		repository = new MultiFileSystem(repList.toArray(new FileSystem[1]));
	}

	/** {@inheritDoc} */
	@Override
	public InputStream getStream(ResourcePackage directory, String name, ResourceType type)
			throws FileNotFoundException, IllegalStateException {
		if (repository == null) {
			throw new IllegalStateException("Repository has not been initialized yet!");
		}

		String fullPath = directory.getName() + PATH_SEPARATOR + name + "." + type.getSuffix();
		FileObject resource = repository.findResource(fullPath);
				
		if (resource == null) {
			throw new FileNotFoundException("Resource " + fullPath + " does not exist!");
		}

		return resource.getInputStream();
	}

	private void openAsZipJarFileFileSystem(File f, List<FileSystem> repList) throws InvalidFileException {
		// create a jar/zip resource
		try {
			JarFileSystem fs = new JarFileSystem();
			fs.setJarFile(f);
			repList.add(fs);
		} catch (Exception e) {
			throw new InvalidFileException(f.getName() + "is not a valid jar or zip file.", e);
		}
	}

	private void openAsFileSystem(File f, List<FileSystem> repList) throws InvalidFileException {
		// Create a file resource
		try {
			LocalFileSystem fs = new LocalFileSystem();
			fs.setRootDirectory(f);
			repList.add(fs);
		} catch (Exception e) {
			throw new InvalidFileException(f.getName() + "cannot be opened.", e);
		}
	}
}
