/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.resourcemanager;


import java.io.FileNotFoundException;
import java.io.InputStream;


/** The interface to repositories.
  *
  * <p>
  * A repository is a class that you can query for a certain resource (i.e.
  * file or file entry in a zip file), and it returns a stream to this file.
  * Typically, a single repository can be the merge of multiple directories.
  * This way, we can have a single directory structure, files lying around in
  * various directories, and you never have to worry about how to access them
  * properly.
  * </p>
  *
  */

public interface ResourceRepository {

	/** Physically sets up the repository.
	  *
	  * This function tries to open the physical paths/files and is required
	  * before you can do anything with the class.
	  *
	  * @throws InvalidFileException if one of the specified file or path names
	  * is invalid.
	  */
	void connectToResources() throws InvalidFileException;

	/** Returns a new input stream from the specified resource/file.
	  *
	  * @param directory 
	  *     an instance of Package that specifies the directory where the
	  *     requested file resides.
	  * @param name
	  *     the name of the requested file
	  * @param type
	  *     an instance of ResourceType that specifies possible suffixes and
	  *     the type of file (sound, graphics, text, ...)
	  * @return an input stream that reads data from the requested file. The
	  *     type of the input stream depends on the specified ResourceType.
	  * @throws InvalidFileException if the resource does not exist.
	  * @throws IllegalStateException if the repository has not been
	  * initialised yet (via connectToResources()).
	  */
	InputStream getStream(ResourcePackage directory, String name, ResourceType type) 
			throws FileNotFoundException, IllegalStateException;

}
