/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.io.InputStream;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;

import org.freelords.sound.command.MutePlayer;
import org.freelords.sound.command.ResumePlayer;
import org.freelords.sound.command.RunPlayer;
import org.freelords.sound.command.SetSoundVolume;
import org.freelords.sound.command.SoundCommand;
import org.freelords.sound.command.StopPlayer;

/** The central class for playing audio.
 *
 * <p>
 * Typically, from the outside, you never have to do with the details of how to
 * start players etc. You only use functions defined in this class. If you start
 * new sounds or music, you sometimes get a reference to the player, which you
 * can use to subsequently kill it if required. 
 * </p>
 *
 * <p>
 * To avoid deadlocks, the class does not manipulate players directly. Instead, it
 * puts all calls into a queue, which is read by an AudioManagerExecutor. This is a bit
 * more complicated, but with sufficient care is guaranteed to be deadlock-safe.
 * Due to possible race conditions, all functions that have to do with event
 * playing are also synchronized (actually, it is only the event player that
 * needs to be accessed in a thread-safe manner); for playing of other sounds,
 * we manipulate a ConcurrentMap for the same reason.
 * </p>
 */

public class AudioManager implements AudioListener {

	/** The mixer that does the sound output. */
	private Mixer mixer;
	/** A queue that we forward events to. */
	private BlockingQueue<SoundCommand> queue;
	/** A sound mapping. */
	private ScenarioEventSounds playlist;
	/** the currently active event player. */
	private AudioPlayer eventPlayer;
	/** The current event to be played. */
	private SoundEvent event;
	/** A mapping of all players that are running. */
	private ConcurrentMap<String, AudioPlayer> soundPlayers;
	/** Whether new events players should be started silently. */
	private boolean muteEvents = false;
	/** Whether new sound players should be started silently. */
	private boolean muteSounds = false;
	/** The volume with which new event players are started. */
	private int eventVolume = 100;
	/** The volume with which new sound players are started. */
	private int soundVolume = 100;

	public AudioManager(Mixer mixer, BlockingQueue<SoundCommand> queue) {
		this.mixer = mixer;
		this.queue = queue;
		soundPlayers = new ConcurrentHashMap<String, AudioPlayer>();
	}

	/** Sets an event mapping that can be used to look up events.
	 *
	 * @param playlist
	 *          the mapping from events to pieces to play.
	 */
	public void setEventSounds(ScenarioEventSounds playlist) {
		this.playlist = playlist;
	}

	/** Plays a sound stream.
	 *
	 * Obtains a new output line and plays the audio file that is supplied as
	 * parameter. This function call does not loop the sound stream.
	 *
	 * @see #playSound(InputStream,boolean,int,boolean) the more general
	 * function that this forwards to.
	 *
	 * @param input
	 *          an input stream that represents an audio file.
	 */
	public String playSound(InputStream input) {
		return playSound(input, false, 0, false);
	}

	/** Plays a sound stream, optionally looping.
	 *
	 * If playing fails, this function silently returns.
	 *
	 * @param input
	 *          an input stream that represents an audio file
	 * @param loop
	 *          determines whether the stream is looped.
	 * @param step
	 *          the gap between loops in milliseconds.
	 * @param randomStep
	 *          if set to true, choose the gap randomly in a uniform manner
	 *          between 0 and step.
	 * @return a reference to the player that can be used to kill the player or
	 *         null if the player could not be set up for some reason.
	 * @throws IllegalArgumentException if the input does not point to a valid
	 *         audio file of if the step parameter is negative.
	 */
	public String playSound(InputStream input, boolean loop, int step, boolean randomStep) {
		if (step < 0) {
			throw new IllegalArgumentException("stepping is negative");
		}
		String ref = UUID.randomUUID().toString();
		try {
			AudioPlayer player = playClip(input, loop, step, randomStep);
			soundPlayers.put(ref, player);
		} catch (LineUnavailableException e) {
			return null;
		}

		return ref;
	}

	/** Plays the sound for a given event.
	 *
	 * If something fails with playing the event, the player dies silently.
	 *
	 * @param event
	 *          the event for which we should play a sound.
	 * @throws IllegalStateException if no event mapping has been set by setEventSounds().
	 */
	public synchronized void playEvent(SoundEvent event) {
		if (playlist == null) {
			throw new IllegalStateException("You must define a mapping for events before using it.");
		}

		this.event = event;

		// stop the previous event.
		removeEvent();

		try {
			InputStream input = playlist.getEntry(event);
			eventPlayer = playClip(input, false, 0, false);
		} catch (Exception e) {
			// doesn't work, so we silently fail.
		}

		// maybe a bit of overkill? However, it should do the job.
		if (muteEvents) {
			stopSystem();
		}
		setEventVolume(eventVolume);
	}

	/** Internal function to play an input stream.
	 *
	 * This does the actual work of setting up a player etc., which is
	 * essentially identical for event and normal sounds.
	 *
	 * @throws LineUnavailableException if no clip could be obtained.
	 * @throws IllegalArgumentException if the input stream is not a valid audio file.
	 */
	private AudioPlayer playClip(InputStream input, boolean loop, int step, boolean randomStep) throws LineUnavailableException {
		AudioInputStream audioIn;
		try {
			audioIn = AudioSystem.getAudioInputStream(input);
		} catch (Exception e) {
			throw new IllegalArgumentException("Invalid sound file.", e);
		}

		// to make sound work, we need to request a specific format that the
		// audio input is converted to and that the clip should support.
		AudioFormat requestedFormat = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100, 16, 2, 4, 44100, false);
		audioIn = AudioSystem.getAudioInputStream(requestedFormat, audioIn);
		Clip clip = (Clip) mixer.getLine(new DataLine.Info(Clip.class, requestedFormat));

		AudioPlayer player = new ClipPlayer(clip, audioIn, loop, step, randomStep);
		player.addAudioListener(this);

		queue.offer(new RunPlayer(player, muteSounds, soundVolume));

		return player;
	}

	/** Callback to loop if an event player has finished playing. */
	@Override
	public synchronized void playerFinished(AudioPlayer player) {
		if (player == eventPlayer) {
			playEvent(event);
		}

		if (soundPlayers.containsValue(player)) {
			for (Map.Entry<String, AudioPlayer> entry : soundPlayers.entrySet()) {
				if (entry.getValue() == player) {
					soundPlayers.remove(entry.getKey());
					break;
				}
			}
		}
	}

	/** Stops a currently playing sound.
	 *
	 * On invalid input, the function silently fails.
	 *
	 * @param reference
	 *          A reference to identify the player; this was returned by playSound().
	 */
	public void removeSound(String reference) {
		if (soundPlayers.containsKey(reference)) {
			final AudioPlayer player = soundPlayers.get(reference);
			soundPlayers.remove(reference);
			queue.offer(new StopPlayer(player));
		}
	}

	/** Stops a currently playing event. */
	public synchronized void removeEvent() {
		if (eventPlayer != null) {
			queue.offer(new StopPlayer(eventPlayer));
			eventPlayer = null;
		}
	}

	/** Stops all playing sounds and events. */
	public void finish() {
		removeEvent();

		while (!soundPlayers.isEmpty()) {
			removeSound(soundPlayers.keySet().iterator().next());
		}
	}

	/** Mutes the playing of all sounds. */
	public void stopSound() {
		muteSounds = true;
		for (AudioPlayer player : soundPlayers.values()) {
			queue.offer(new MutePlayer(player));
		}
	}

	/** Mutes all players (sounds and events) */
	public synchronized void stopSystem() {
		stopSound();
		if (eventPlayer != null) {
			queue.offer(new MutePlayer(eventPlayer));
		}
		muteEvents = true;
	}

	/** Unmutes everything. */
	public synchronized void resumeSystem() {
		muteSounds = false;
		muteEvents = false;

		for (AudioPlayer player : soundPlayers.values()) {
			queue.offer(new ResumePlayer(player));
		}

		if (eventPlayer != null) {
			queue.offer(new ResumePlayer(eventPlayer));
		}
	}

	/** Sets the volume of the event player. */
	public synchronized void setEventVolume(final int volume) {
		eventVolume = volume;

		if (eventPlayer != null) {
			queue.offer(new SetSoundVolume(volume, eventPlayer));
		}
	}

	/** Sets the volume of all normal sound players. */
	public void setSoundVolume(final int volume) {
		soundVolume = volume;
		for (AudioPlayer player : soundPlayers.values()) {
			queue.offer(new SetSoundVolume(volume, player));
		}
	}
}
