/*
 * Copyright (C) 2011 Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.BooleanControl;
import javax.sound.sampled.Clip;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;

/** An AudioPlayer that wraps around a clip.
 *
 * It takes a clip and a sound file as input, and just plays it. Playing clips
 * does not block execution, this has been verified with one of the sound tests.
 */

public class ClipPlayer implements AudioPlayer, LineListener {

	/** The clip to play. */
	private Clip clip;

	/** The sound stream to play. */
	private AudioInputStream input;

	/** A list of listeners to be notified when playing stops. */
	private List<AudioListener> listeners;

	/** whether to loop or not. */
	private boolean loop;

	/** the length of the gap between looping in ms. */
	private int stepping;

	/** if set, choose some random number between 0 and stepping for the gap. */
	private boolean randomStepping;

	/** A random number generator. */
	private static Random rng = new Random();


	/** Constructs a player without looping.
	 *
	 * @param clip
	 *          a clip that is used to play the sounds.
	 * @param input
	 *          a stream that supplies the audio data.
	 */
	public ClipPlayer(Clip clip, AudioInputStream input) {
		this(clip, input, false, 0, false);
	}
   
	/** Constructs a player that can loop if requested.
	 *
	 * @param clip
	 *          a clip that is used to play the sounds.
	 * @param input
	 *          a stream that supplies the audio data.
	 * @param loop
	 *          if true, loop the clip over and over again.
	 * @param stepping
	 *          the gap of silence between loops in milliseconds.
	 * @param random
	 *          if true, the gap is a random number between 0 and the value of stepping.
	 */
	public ClipPlayer(Clip clip, AudioInputStream input, boolean loop, int stepping, boolean random) {
		this.clip = clip;
		this.input = input;

		this.loop = loop;
		this.stepping = stepping;
		this.randomStepping = random;

		listeners = new ArrayList<AudioListener>();
	}

	/** Starts playback.
	  *
	  * Usually, this function is called by passing the player to a separate
	  * thread, because the opening of the clip/reading in of the sound data
	  * takes a while, which would block the main thread for too long.
	  */
	public synchronized void start() {
		try {
			clip.open(input);
			clip.start();
			clip.addLineListener(this);
		}
		catch (Exception e) {
			throw new IllegalStateException("Could not open line", e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void mute() {
		try {
			BooleanControl muteControl = (BooleanControl)clip.getControl(BooleanControl.Type.MUTE);
			muteControl.setValue(true);
		}
		catch (Exception e) {
			throw new UnsupportedOperationException("Clip cannot be silenced.", e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void resume() {
		try {
			BooleanControl muteControl = (BooleanControl)clip.getControl(BooleanControl.Type.MUTE);
			muteControl.setValue(false);
		}
		catch (Exception e) {
			throw new UnsupportedOperationException("Clip cannot be unsilenced.", e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void setVolume(int volume) {
		if (volume > 100 || volume < 0) {
			throw new IllegalArgumentException("Volume must be in the range 0 to 100");
		}

		try {
			FloatControl volumeControl = (FloatControl)clip.getControl(FloatControl.Type.MASTER_GAIN);

			// note: the volume specifies the amplitude of the signal in a
			// linear fashion (volume 50 means half of the maximum amplitude), while
			// the master gain specifies the volume in dB, where the conversion is
			// amplitude = 10**(dB/20)
			// So we need to do a bit of math for the conversion.
			float dB = new Double(20*Math.log10(volume/100.0) + volumeControl.getMaximum()).floatValue();
			dB = Math.max(dB, volumeControl.getMinimum());

			volumeControl.setValue(dB);
		}
		catch (Exception e) {
			throw new UnsupportedOperationException("Master gain control not available.", e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void stop() {
		// stopping the clip calls the callback function.
		loop = false;
		clip.stop();
	}

	/** {@inheritDoc} */
	@Override
	public synchronized void addAudioListener(AudioListener listener) {
		listeners.add(listener);
	}

	@Override
	public synchronized void removeAudioListener(AudioListener listener) {
		listeners.remove(listener);
	}

	/** Callback when the status of the clip changes.
	 *
	 * We mostly wait for the clip to stop because it has run out of sound
	 * data. If this happens, the player cleans up all transient data and
	 * is invalidated. If the player has been set to looping, the player will
	 * be restarted.
	 */
	@Override
	public synchronized void update(LineEvent event) {
		if (event.getType() != LineEvent.Type.STOP) {
			return;
		}

		// if looping is set, sleep for some time and restart the sound
		if (loop) {
			int sleepingTime;
			if (randomStepping) {
				sleepingTime = rng.nextInt(stepping);
			}
			else {
				sleepingTime = stepping;
			}

			try {
				Thread.sleep(sleepingTime);
			}
			catch (InterruptedException e) {
				// well, if we cannot send the thread to sleep, then there will
				// be no break...
			}

			clip.setFramePosition(0);
			clip.start();
			return;
		}


		// otherwise, clean up.

		// notify listeners
		for (AudioListener listen : listeners) {
			listen.playerFinished(this);
		}

		// clean up
		try {
			clip.close();
			input.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
