/*
 * Copyright (C) 2011 Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

import org.freelords.resourcemanager.ResourceRepository;
import org.freelords.resourcemanager.ResourceType;
import org.freelords.sound.jaxb.XmlHashMapAdapter;

/** A mapping from events to sound/music files.
 *
 * <p>
 * Usually, this class is loaded from an XML file for a given scenario by the
 * server and sent to the client. So it needs to be loadable from XML, and it
 * needs to be serializable. On top of that, it should offer validity checks
 * for convenience, and it should also return the strings, of course. Note
 * that the class is purely read-only. I cannot really imagine a case where
 * you would want to modify what sounds to play after initialization.
 * </p>
 *
 * <p>
 * Note that JAXB does not contain a native binding to HashMaps, we have to
 * work around with custom-made classes and adapters. See the documentation of
 * {@link javax.bind.annotation.adapters.XmlAdapter} for the details.
 * </p>
 *
 */
@XmlRootElement(name = "event-sounds")
public class ScenarioEventSounds implements Serializable {

	private static final long serialVersionUID = 1510855646881539164L;
	/** The mapping from events to sound files. */
	@XmlElement
	@XmlJavaTypeAdapter(XmlHashMapAdapter.class)
	private Map<SoundEvent, List<SoundMapping>> mapping;
	/** A Repository to look up files. */
	private transient ResourceRepository rep;
	/** Some random number generator. */
	private transient Random randomNumberGenerator;

	/** The constructor does not have much to do. */
	public ScenarioEventSounds() {}

	/** Sets the repository to use for lookup, and checks for existence of all files.
	 *
	 * @param rep
	 *          a resource repository that can resolve all input stream requests.
	 * 
	 * @throws FileNotFoundException if any of the sound files are not available.
	 */
	public void setResourceRepository(ResourceRepository rep) throws FileNotFoundException {
		this.rep = rep;

		// Check that all files exist. The repository will throw an error if not.
		for (List<SoundMapping> entrylist : mapping.values()) {
			for (SoundMapping entry : entrylist) {
				this.rep.getStream(entry.getDirectory(), entry.getFile(), ResourceType.SOUND);
			}
		}
		
		// Since this is a required initialisation procedure, we set the random
		// number generator here.
		randomNumberGenerator = new Random();
	}

	/** Get a random entry for a given event.
	 *
	 * @param key
	 *         the event for which we want to get a music piece.
	 * @return a random entry for this event
	 * @throws IllegalArgumentException if there is no entry for the key.
	 * @throws IOException if the sound file cannot be found. This exception
	 * indicates an invalid repository.
	 */
	public InputStream getEntry(SoundEvent key) throws IOException {
		if (!mapping.containsKey(key)) {
			throw new IllegalArgumentException("No sound defined for " + key);
		}

		List<SoundMapping> entrylist = mapping.get(key);
		SoundMapping entry = entrylist.get(randomNumberGenerator.nextInt(entrylist.size()));
		try {
			return new BufferedInputStream(
					rep.getStream(entry.getDirectory(), entry.getFile(), ResourceType.SOUND));
		} catch (FileNotFoundException e) {
			throw new IOException(e);
		}
	}	
}
