/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz, Ion Pana
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

/** Facilitates the marshaling and unmarshaling of objects by hiding some details.
 *
 * This is used to initialize the xml Marshall/Unmarshall system. First of all
 * you register the JaxB annotated class within the manager, then call the
 * initXmlSystem() method and you are ready to marshall/unmarshall your class.
 */
public class XmlManager {

	private Unmarshaller unmarshaller;
	private Marshaller marshaller;
	private List<Class> classList = new ArrayList<Class>();

	/**
	 * Registers a new class to be marshalled or unmarshalled.
	 * 
	 * @param type  The type of the Class to be marshalled/unmarshalled 
	 */
	public void registerClass(Class type) {
		this.classList.add(type);
	}

	/**
	 * Initializes the xml system.
	 *
	 * Creates a Marshaller and an unmarshaller, and does the basic xml setup. 
	 * 
	 * @throws JAXBException if initialization does not work. 
	 */
	public void initXmlSystem() throws JAXBException {
		JAXBContext jc = JAXBContext.newInstance(classList.toArray(new Class[0]));

		unmarshaller = jc.createUnmarshaller();
		marshaller = jc.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		marshaller.setEventHandler(new javax.xml.bind.helpers.DefaultValidationEventHandler());
	}

	/**
	 * Marshal objects into xml files.
	 *
	 * The xml file structure is created from the object's structure. This is
	 * done by annotating the object's class with the usual JAXB annotations.
	 * 
	 * @param object The Object to be marshalled
	 * @param out the output stream containing the marshalled xml   
	 * @throws JAXBException If something goes wrong during marshalling.
	 */
	public <T> void marshal(T object, ByteArrayOutputStream out) throws JAXBException {
		marshaller.marshal(object, out);
	}

	/**
	 * Unmarshal ojects from xml files.
	 *
	 * The xml file structure must match the object's structure. This is done
	 * annotating the object's class.
	 * 
	 * @param xmlFile  The xml file from which we want to create a new object.
	 * @return the unmarshalled object.
	 * @throws JAXBException If something goes wrong during unmarshalling.
	 */
	@SuppressWarnings("unchecked")
	public <T> T unmarshall(File xmlFile) throws JAXBException {
		return (T) unmarshaller.unmarshal(xmlFile);
	}
	
	/**
	 * Unmarshal objects from xml streams.
	 *
	 * The xml structure must match the object's structure. This is done
	 * annotating the object's class.
	 * 
	 * @param xmlStream The xml stream from which we want to create a new object.
	 * @return the unmarshalled object.
	 * @throws JAXBException If something goes wrong during unmarshalling.
	 */
	@SuppressWarnings("unchecked")
	public <T> T unmarshall(InputStream xmlStream) throws JAXBException {
		return (T) unmarshaller.unmarshal(xmlStream);
	}
}
