/*
 * Copyright (C) 2011 Leonel Iturralde, Andrea Paternesi, Ulf Lorenz, Premkumar Bhaskal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import org.freelords.map.Position;
import org.freelords.map.tile.Access;
import org.freelords.army.exception.ArmyTooBigException;
import org.freelords.army.exception.UnitNotInTheArmyException;
import org.freelords.army.exception.ArmyIsNotSplittableException;
import java.util.ArrayList;
import java.util.List;
import org.junit.Before;
import org.junit.Test;
import org.freelords.map.tile.MapTile;

import static org.junit.Assert.*;

public class ArmyTest {

	private String name = "TEST";
	private int movementpoints = 10;
	private Access access = Access.WALKABLE;
	private MapTile maptile;
	private Position testPosition = new Position(4, 5);
	private List<Unit> emptyUnitList = new ArrayList<Unit>();
	private List<Unit> oneUnitList = new ArrayList<Unit>();
	private List<Unit> fullUnitsList = new ArrayList<Unit>();
	private Unit singleunit = new Unit();
	private Army oneUnitArmy;
	private Army fullUnitArmy;
	private final Integer MINIMUM_MP = 5;

	@Before
	public void setUp() throws Exception {
		singleunit.setMovementPoints(MINIMUM_MP);

		oneUnitList.add(singleunit);
		fullUnitsList.add(singleunit);

		// fill the list with the remaining armies; set movement points for some
		// test.
		for (int i = 0; i < Army.MAX_SIZE - 1; i++) {
			Unit unit = new Unit();
			unit.setMovementPoints(MINIMUM_MP + i);

			fullUnitsList.add(unit);
		}

		maptile = new MapTile(name, testPosition, movementpoints, access);

		oneUnitArmy = new Army(oneUnitList, maptile);
		fullUnitArmy = new Army(fullUnitsList, maptile);
	}

	@Test(expected = IllegalArgumentException.class)
	public void creatingAnArmywithanEmptyUnitListThrowsAnException()
			throws Exception {
		new Army(emptyUnitList, maptile);
	}

	@Test(expected = ArmyTooBigException.class)
	public void creatingAnArmywithABigUnitListThrowsAnException()
			throws Exception {
		// add one more unit to an already full list.
		fullUnitsList.add(new Unit());

		new Army(fullUnitsList, maptile);
	}

	@Test
	public void theArmyConstructorSetParametersCorrectly() {
		assertEquals("The unit list size was not what was set.",
				oneUnitList.size(), oneUnitArmy.getUnits().size());
		assertEquals("The unit list differs from what was set.",
				oneUnitList.get(0), oneUnitArmy.getUnits().get(0));
		assertEquals("The position was wrong.", testPosition,
				oneUnitArmy.getPosition());
	}

	@Test
	public void checkIfArmyisSplittable() {
		assertFalse("An army with a single unit must not be splittable.",
				oneUnitArmy.isSplittable());
		assertTrue("An army with multiple units should be splittable.",
				fullUnitArmy.isSplittable());
	}

	@Test
	public void checkIfArmyisMergeable() throws Exception {
		assertTrue("Two valid armies must be mergeable.",
				oneUnitArmy.isMergeable(new Army(oneUnitList, maptile)));
		assertFalse("Two armies with too many units must not be mergeable.",
				oneUnitArmy.isMergeable(fullUnitArmy));

		fullUnitsList.remove(0);
		fullUnitArmy = new Army(fullUnitsList, maptile);
		assertTrue(
				"Two valid armies of maximum allowed size must be mergeable.",
				oneUnitArmy.isMergeable(fullUnitArmy));
	}

	@Test
	public void checkThatArmiesMergedCorrectly() throws Exception {
		oneUnitArmy.merge(new Army(oneUnitList, maptile));
		oneUnitList.addAll(oneUnitList);
		assertEquals("The merge did not work.", oneUnitList,
				oneUnitArmy.getUnits());
	}

	@Test(expected = ArmyTooBigException.class)
	public void MergingTwoArmyThatAreTooBigThrowsAnException() throws Exception {
		oneUnitArmy.merge(fullUnitArmy);
	}

	@Test
	public void theArmyWasSplitCorrectly() throws Exception {
		int bigArmySizeBefore = fullUnitArmy.getUnits().size();
		Army splittedArmy = fullUnitArmy.split(oneUnitList);

		assertEquals("Army was not correctly splitted.",
				oneUnitArmy.getUnits(), splittedArmy.getUnits());
		assertEquals("The position of the splitted army was wrong.",
				fullUnitArmy.getPosition(), splittedArmy.getPosition());
		assertEquals("The size of the remaining Army is not correct.",
				bigArmySizeBefore - 1, fullUnitArmy.getUnits().size());
	}

	@Test(expected = UnitNotInTheArmyException.class)
	public void theSplitMethodThrowsAnExceptionIfUnitListNotInTheArmy()
			throws Exception {
		Unit armyNotInArmy = new Unit();
		List<Unit> listThatContainsArmyNotInArmy = new ArrayList<Unit>();
		listThatContainsArmyNotInArmy.add(armyNotInArmy);

		fullUnitArmy.split(listThatContainsArmyNotInArmy);
	}

	@Test(expected = ArmyIsNotSplittableException.class)
	public void theSplitMethodThrowsAnExceptionIfArmyIsNotSplittable()
			throws Exception {
		oneUnitArmy.split(oneUnitList);
	}

	@Test(expected = IllegalArgumentException.class)
	public void theSplitMethodThrowsAnExceptionIfTheListIsEmpty()
			throws Exception {
		fullUnitArmy.split(emptyUnitList);
	}

	@Test
	public void checkedIfGettingMovementPoints() {
		assertEquals("Army does not return the Minimun Movement Points",
				MINIMUM_MP, fullUnitArmy.getMovementPoints());
	}

	@Test
	public void checkedIfArmiesAreRemovedCorrectly() throws Exception {
		fullUnitArmy.removeUnits(oneUnitList);
		fullUnitsList.removeAll(oneUnitList);
		assertEquals("The army did not correctly remove the units", new Army(
				fullUnitsList, maptile).getUnits(), fullUnitArmy.getUnits());
	}

	@Test(expected = IllegalArgumentException.class)
	public void theRemoveMethodThrowsExceptionWhenArmyStackIsEmptyAfterRemoval()
			throws Exception {
		oneUnitArmy.removeUnits(oneUnitList);
	}

	@Test(expected = UnitNotInTheArmyException.class)
	public void theRemoveMethodThrowsExceptionForEmptyLists() throws Exception {
		// Note: Also throws the exception if the unitList is empty
		// because the units(even though there arent) are not within the Army
		fullUnitArmy.removeUnits(emptyUnitList);
	}

	@Test(expected = UnitNotInTheArmyException.class)
	public void theRemoveMethodThrowsExceptionForInvalidArmies()
			throws Exception {
		fullUnitArmy.removeUnits(oneUnitList);
		fullUnitArmy.removeUnits(oneUnitList);
	}
}
