/*
 * Copyright (C) 2011 Ion Pana, Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import static junit.framework.Assert.*;

import java.util.HashSet;
import java.util.Set;

import org.freelords.army.exception.NoSuchUnitException;
import org.freelords.army.exception.ValidationException;

import org.junit.Before;
import org.junit.Test;

public class UnitManagerTest {

	private static final String GOOD_UNIT = "goodUnit";
	private static final String BAD_UNIT = "badUnit";
	private static final String NO_SUCHUNIT = "noSuchUnit";
	private static final int UNIQUE_THRESHOLD = 2000;

	private UnitManager unitManager;
   
	@Before
	public void setUp() throws Exception {
		this.unitManager = UnitManagerHelper.createTestUnitManager();
	}

	/**
	 * Test that a unit is created out of a well defined file and 
	 * if the created unit passes validation rules.
	 */
	@Test
	public void testCreateWellDefinedUnit() throws Exception {
		Unit unit = this.unitManager.createUnit(GOOD_UNIT);
		//unit is not null and valid
		assertNotNull("The unit was null.", unit);
		validateUnit(unit);
	}

	/**
	 * Test that a ValidationException is thrown when trying to create an 
	 * unit out of a faulty definition file.
	 */
	@Test(expected = ValidationException.class)
	public void testCreateBadlyDefinedUnitThrowsValidationException() throws Exception {
		this.unitManager.createUnit(BAD_UNIT);
	}

	/**
	 * Test if NoSuchUnitException is thrown when trying to create a non-existent unit.
	 */
	@Test(expected = NoSuchUnitException.class)
	public void testCreateNonExistentUnitThrowsException() throws Exception {
		this.unitManager.createUnit(NO_SUCHUNIT);
	}

	/**
	 * Validates a unit according to the following rules.
	 * 
	 * Movement Points &gt;= 0
	 * 
	 * @param unit the unit to be validated
	 */
	private void validateUnit(Unit unit) {
		assertTrue("Movement points were < 0.", unit.getMovementPoints() >= 0);
	}

	/**
	 * Tests that a created unit gets a non-null id.
	 * @throws Exception
	 */
	@Test
	public void testUnitIdIsCorrectlySetUp() throws Exception {
		Unit unit = this.unitManager.createUnit(GOOD_UNIT);
		assertNotNull("The unit id was null.", unit.getId());
	}

	/**
	 * Test to validate "uniqueness". It creates a number (2000) of armies out of the same definition
	 * and puts the ids in a Set. Validates that the Set contains the same number in the end (that is, 
	 * all the generated ids were unique).
	 * @throws Exception
	 */
	@Test
	public void testUnitIdIsUnique() throws Exception {
		Set<String> uniqueIds = new HashSet<String>();
		//put a lot of ids in the set
		for (int i = 0; i < UNIQUE_THRESHOLD; i++) {
			Unit unit = this.unitManager.createUnit(GOOD_UNIT);
			uniqueIds.add(unit.getId());
		}
		//and validate the size of the set is equal to the amount
		assertEquals("The set size was different from expected : at least an unit id was not unique.", UNIQUE_THRESHOLD, uniqueIds.size());
	}
}
