/*
 * Copyright (C) 2011 Pana Ion, Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army;

import static junit.framework.Assert.assertEquals;

import java.io.File;

import org.freelords.xmlmanager.utils.XPathHelper;
import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import org.junit.Before;
import org.junit.Test;

public class UnitStatCalculatorModifierIntegrationTest {

	private File modifierFile = new File(this.getClass().getResource("files/integrationModifier.xml").getFile());
	private Modifier modifier;
	private Unit unit;
	private int maximumMovePoints;
	private int upkeep;
	
	@Before
	public void setUp() throws Exception {
		// initialize XmlManager and load a modifier
		XmlManager xmlManager = XmlTestHelper.createXmlManagerFor(Unit.class, Modifier.class);
		this.modifier = xmlManager.unmarshall(modifierFile);
	   
		// initialize a unit manager and load a unit
		UnitManager unitManager = UnitManagerHelper.createTestUnitManager();
		this.unit = unitManager.createUnit("integrationUnit");
		
		//get the actual values inside the xml
		XPathHelper xPathHelper = new XPathHelper(new File(this.getClass().getResource("files/repository/army/integrationUnit.xml").getFile()));
	   
		maximumMovePoints = xPathHelper.evaluateXPathAsInteger("//stats/entry[@stat='MAXIMUM_MOVE_POINTS']/@value");
		upkeep = xPathHelper.evaluateXPathAsInteger("//stat-calculator/stats/entry[@stat='UPKEEP']/@value");
	}

	@Test
	public void testGetUnitStat() {
		//test the raw stats are correctly read and calculated
		validateRawStats();
	}

	@Test
	public void testAddModifier() throws Exception {
		//an additional modifier will have an additive effect on the raw stats
		unit.addModifier(this.modifier);

		validateRawStatsWithModifier();
	}

	@Test(expected = IllegalArgumentException.class)
	public void testAddSameModifierTwiceIsNotAllowed() {
		unit.addModifier(this.modifier);
		unit.addModifier(this.modifier);
	}

	@Test
	public void testRemoveModifier() {
		//adding and removing the same modifier should restore the original raw stats
		unit.addModifier(this.modifier);
		validateRawStatsWithModifier();

		unit.removeModifier(this.modifier);
		validateRawStats();
	}

	@Test
	public void testRemoveANonExistingModifierFailsSilently() {
		//the modifier is not there, and no exception must be thrown
		unit.removeModifier(this.modifier);
	}

	private void validateRawStats() {
		assertEquals("Raw maximum move points incorrect.", 
				maximumMovePoints, this.unit.getUnitStat(UnitStat.MAXIMUM_MOVE_POINTS));
		assertEquals("Raw upkeep incorrect.", upkeep, this.unit.getUnitStat(UnitStat.UPKEEP));
	}

	private void validateRawStatsWithModifier() {
		assertEquals("Maximum move points modified value is incorrect.", 
				maximumMovePoints + this.modifier.getStatValue(UnitStat.MAXIMUM_MOVE_POINTS),
				this.unit.getUnitStat(UnitStat.MAXIMUM_MOVE_POINTS));
		assertEquals("Upkeep modified value is incorrect.", 
				upkeep + this.modifier.getStatValue(UnitStat.UPKEEP),
				this.unit.getUnitStat(UnitStat.UPKEEP));
	}
}
