/*
 * Copyright (C) 2011 Paul Statham
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.army.heroes;

import static org.junit.Assert.*;

import java.io.File;

import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import org.junit.Before;
import org.junit.Test;

public class BackPackTest {
	
	private File itemFile = new File(this.getClass().getResource("files/item.xml").getFile());
	private Item item;
	private Backpack testBackPack;
	private Equipment equipment;
	
	@Before
	public void setUp() throws Exception {
		XmlManager itemManager = XmlTestHelper.createXmlManagerFor(Item.class);
		this.item = itemManager.unmarshall(this.itemFile);
		this.testBackPack = new Backpack();
		this.equipment = new Equipment();
	}
	
	/* Test removing an object that is not in the back pack */
	@Test(expected = IllegalArgumentException.class)
	public void testRemoveItemThrowsExceptionIfItemNotInBackPack() {
		testBackPack.removeItem(item);
	}

	/* Test that removing works fine after storing the object in the backpack */
	@Test
	public void testStoreAndRemoveItem() {
		testBackPack.storeItem(item);
		testBackPack.removeItem(item);
	}

	/* Removing an item twice from the backpack throws an exception again. */
	@Test(expected = IllegalArgumentException.class)
	public void removingCleansTheBackpack() {
		testBackPack.storeItem(item);
		testBackPack.removeItem(item);
		testBackPack.removeItem(item);
	}
	
	/* Test equipping an item in the backpack */
	@Test
	public void testEquipItem() {
		setupEquipment();		
		Item removedItem = equipment.remove(item.getType());
		assertEquals("Items are not equal!", removedItem, item);
	}
	
	/* Test equipping an item that is not in the backpack */
	@Test(expected = IllegalArgumentException.class)
	public void testEquipItemNotInBackPack() {		
		testBackPack.equipItem(equipment, item);
	}
	
	/* Test removing an item from the equipment */
	@Test
	public void testTakeOffItem() {
		setupEquipment();
		testBackPack.takeOffItem(equipment, item.getType());
		assertNull("Item not removed from equipment!", equipment.remove(item.getType()));

		// check that the item has been added to the backpack. If not, the
		// following line throws an exception.
		testBackPack.removeItem(item);
	}
	
	/* Test removing an item from an empty slot */
	@Test(expected = IllegalArgumentException.class)
	public void testTakeOffItemWhenItemNotEquipped() {		
		testBackPack.takeOffItem(equipment, ItemType.ARMOR);
	}
	
	private void setupEquipment() {		
		testBackPack.storeItem(item);
		testBackPack.equipItem(equipment, item);
	}

}
