/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.client;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.freelords.network.client.fake.FakeKryonetClient;
import org.freelords.network.fake.FakeNetworkCommand;
import org.junit.Before;
import org.junit.Test;

import com.esotericsoftware.kryonet.Listener;

public class FLNetworkClientTest {

	private static final String SERVER_ADDRESS = "127.0.0.1";
	private static final Integer SERVER_PORT = 9999;
	private FakeKryonetClient fakeKryonetClient = new FakeKryonetClient();
	private Session session = new Session();
	private FLNetworkClient client;
	private FLNetworkClient unconfiguredClient;

	@Before
	public void setUp() throws Exception {
   		client = new FLNetworkClient(fakeKryonetClient, session);
		client.registerClass(FakeNetworkCommand.class);
		client.addReceivingListener(new Listener());
		client.initNetwork();

		unconfiguredClient = new FLNetworkClient(fakeKryonetClient, session);
	}

	@Test(expected = IllegalArgumentException.class)
	public void createClientWithNoSessionObject() throws Exception {
		new FLNetworkClient(fakeKryonetClient, null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void createClientWithNoKryonetObject() throws Exception {
		new FLNetworkClient(null, session);
	}

	@Test(expected = IOException.class)
	public void startUnconfiguredClient() throws IOException {
		assertFalse("Client was running before starting it.", fakeKryonetClient.isRunning);
		unconfiguredClient.start();
		assertFalse("Unconfigured Client was running.", fakeKryonetClient.isRunning);
	}

	@Test(expected = IOException.class)
	public void connectUnconfiguredClient() throws IOException {
		assertFalse("Client was connected before configuring it.", fakeKryonetClient.isConnected);
		unconfiguredClient.connect(SERVER_ADDRESS,SERVER_PORT);
		assertFalse("Unconfigured Client was connected.", fakeKryonetClient.isConnected);
	}
	
	@Test(expected = IOException.class)
	public void connectNotStartedConfiguredClient() throws IOException {
		assertFalse("Client was connected before configuring it.", fakeKryonetClient.isConnected);
		client.connect(SERVER_ADDRESS,SERVER_PORT);
		assertFalse("Unconfigured Client was connected.", fakeKryonetClient.isConnected);
	}
	
	@Test(expected = IOException.class)
	public void stopNotRunningClient() throws IOException {
		unconfiguredClient.stop();
	}

	@Test(expected = IOException.class)
	public void configureClientWithoutAddingNetworkCommands() throws IOException {
		unconfiguredClient.addReceivingListener(new Listener());
		unconfiguredClient.initNetwork();
	}

	@Test
	public void configureClientCorrectly() throws IOException {
		assertTrue("The class was not registered correctly.", client.registeredClasses.contains(FakeNetworkCommand.class));
		assertEquals("Client did not register the correct number of classes.", 1, fakeKryonetClient.kryo.registerCalled);
		assertTrue("Client did not register a listener.", fakeKryonetClient.listenerAdded);
	}

	@Test
	public void startWellConfiguredClient() throws IOException {
		assertFalse("Client was running before starting it.", fakeKryonetClient.isRunning);
		client.start();
		assertTrue("Kryonet Client was not running.", fakeKryonetClient.isRunning);
	}

	@Test
	public void connectWellConfiguredAndStartedClient() throws IOException {
		client.start();
		assertFalse("Client was connected before connecting it.", fakeKryonetClient.isConnected);
		client.connect(SERVER_ADDRESS,SERVER_PORT);
		assertTrue("Kryonet Client was not connected.", fakeKryonetClient.isConnected);
	}
	
	@Test(expected = IOException.class)
	public void avoidStartingClientTwice() throws IOException {
		client.start();
		client.start();
	}

	@Test
	public void stopRunningWellConfiguredClient() throws IOException {
		assertFalse("Client was running before starting it.", fakeKryonetClient.isRunning);
		client.start();
		assertTrue("Kryonet Client was not running.", fakeKryonetClient.isRunning);
		client.stop();
		assertFalse("Kryonet Client was not stopped.", fakeKryonetClient.isRunning);
	}

	@Test
	public void sendNetworkCommandoverNetwork() throws IOException {
		FakeNetworkCommand command = new FakeNetworkCommand();
		client.sendCommand(command);
		assertTrue("Command was not sent trough the network.",fakeKryonetClient.commandSent);
	}
}
