/*
 * Copyright (C) Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.command;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

public class ChatCommandsTest {

	private static final String CHAT_MESSAGE = "Chat Message";
	private Set<String> nullClientIdList;
	private Set<String> emptyClientIdList = new HashSet<String>();
	private Set<String> listWithANullClientId = new HashSet<String>();
	private Set<String> goodClientList = new HashSet<String>();

	@Before
	public void setup() {
		listWithANullClientId.add(UUID.randomUUID().toString());
		listWithANullClientId.add(UUID.randomUUID().toString());
		listWithANullClientId.add(null);
		goodClientList.add(UUID.randomUUID().toString());
		goodClientList.add(UUID.randomUUID().toString());
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void chatBroadcastCommandContructorCannotGetANullMessage() {
		new ChatBroadcastCommand(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void chatMulticastCommandContructorCannotGetANullMessage() {
		new ChatMulticastCommand(null, nullClientIdList);
	}

	@Test(expected = IllegalArgumentException.class)
	public void chatMulticastCommandContructorCannotGetANullClientIdList() {
		new ChatMulticastCommand(CHAT_MESSAGE, null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void chatMulticastCommandContructorCannotGetAnEmptyClientIdList() {
		new ChatMulticastCommand(CHAT_MESSAGE, emptyClientIdList );
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void chatMulticastCommandContructorCannotGetNullParameters() {
		new ChatMulticastCommand(null, null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void chatMulticastCommandAllPassedIdsCannotBenNull() {
		new ChatMulticastCommand(CHAT_MESSAGE, listWithANullClientId);
	}
	
	@Test
	public void chatMessageIsCorrectlyAssignedToCommand() {
		ChatBroadcastCommand command = new ChatBroadcastCommand(CHAT_MESSAGE);
		assertEquals("The message was different.", CHAT_MESSAGE, command.getChatText());
	}
	
	@Test(expected = RuntimeException.class)
	public void chatMultiCastCommandReturnsUnmodifiableList() {
		ChatMulticastCommand command = new ChatMulticastCommand(CHAT_MESSAGE, goodClientList);
		Set<UUID> myList = command.getClientIds();
		//the returned list should be unmodifiable
		myList.add(UUID.randomUUID());
	}
	
	@Test
	public void chatMulticastCommandIsCorrectlyConstructed() {
		ChatMulticastCommand command = new ChatMulticastCommand(CHAT_MESSAGE, goodClientList);
		Set<UUID> myList = command.getClientIds();
		assertEquals("There should be 2 elements in the client list", 2, myList.size());
		assertEquals("The message was different.", CHAT_MESSAGE, command.getChatText());
	}
}
