/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz, Ion Pana, Paul Statham
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server;


import java.util.UUID;

import org.freelords.network.command.ClientNetworkRegisterCommand;
import org.freelords.network.common.ClientRole;
import org.freelords.network.fake.FakeFLNetworkConnection;
import org.freelords.network.fake.FakeNetworkCommand;
import org.freelords.network.server.fake.FakeKryonetServer;
import org.freelords.network.server.fake.FakeSecurityManager;
import org.freelords.network.server.fake.FakeService;
import org.freelords.network.server.security.FLSecurityManager;
import org.freelords.network.server.service.ServerClientRegisterService;
import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

public class FLNetworkServerListenerTest {

	private FLSecurityManager securityManager = new FLSecurityManager();
	private FakeFLNetworkConnection connection = new FakeFLNetworkConnection();
	private Updater updater = null;
	private FLNetworkServerListener listener = new FLNetworkServerListener(securityManager, updater);

	private FakeNetworkCommand fakeCommand = new FakeNetworkCommand();
	private ClientNetworkRegisterCommand registerCommand = new ClientNetworkRegisterCommand("test name");
	
	//service for executing fake commands
	private FakeService fakeCommandService = new FakeService();
	
	private FakeKryonetServer fakeServer = new FakeKryonetServer();
	
	@Before
	public void setUp() throws Exception {
		// register a handful of services.
		listener.registerService(fakeCommandService);
		listener.registerService(new ServerClientRegisterService());

		updater = new Updater(fakeServer);

		connection.setClientName("some client name");
		connection.setClientId(UUID.randomUUID());
		connection.setRole(ClientRole.PLAYER);
	}

	@Test
	/**
	 * Test Service throwing an exception from execute
	 */
	public void testServiceThrowingExceptionFromExecute() {
		listener = new FLNetworkServerListener(securityManager, updater);
		listener.registerService(fakeCommandService);
		
		fakeServer.addConnection(connection);
		fakeCommandService.throwException = true;
		
		listener.received(connection, fakeCommand);
		assertTrue("Update not received by connection", connection.isCalled());
	}	
	
	@Test
	/**
	 * Test to ensure that the fake service handles fake commands.
	 */
	public void testFakeService() throws Exception {
		assertTrue("Fake command should be accepted.", fakeCommandService.accept(fakeCommand));
		fakeCommandService.execute(fakeCommand, connection, null);
		assertTrue("Fake service should be executed.", fakeCommandService.executed);
	}
	
	@Test
	/**
	 * Test to see that registering a fakeCommandService to the listener will result in the
	 * proper FakeCommand handling.
	 */
	public void testRegisterServiceOnListener() {
		listener.received(connection, fakeCommand);
		
		assertTrue("Fake service should have been executed.", fakeCommandService.executed);
	}
	
	@Test
	public void testUnregisterServiceOnListener() {
		listener.unregisterService(fakeCommandService);
		listener.received(connection, fakeCommand);
		
		assertFalse("Fake service should not have been executed.", fakeCommandService.executed);
	}
	
	@Test
	public void testUnregisterServiceShouldFailSilently() {
		listener.unregisterService(fakeCommandService);
		listener.unregisterService(fakeCommandService);
	}
	
	@Test
	/**
	 * Multiple services that accept the same command should run.
	 */
	public void testRegisterAndReceiveMultipleServices() {
		FakeService service2 = new FakeService();
		listener.registerService(service2);
		listener.received(connection, fakeCommand);
		
		assertTrue("Fake service should both be executed.", fakeCommandService.executed);
		assertTrue("Fake service should both be executed.", service2.executed);
	}
	
	@Test
	public void testServiceThatDoesNotAcceptDoesNotRun() {
		fakeCommandService.accept = false;
		listener.received(connection, fakeCommand);
		
		assertFalse("Service should not have been executed.", fakeCommandService.executed);
	}
	
	@Test
	/**
	 * If one of the services throws exceptions during processing, it should not affect other services.
	 */
	public void testCrashingServiceDoesNotGetInOtherServicesWay() {
		FakeService errorService = new FakeService();
		errorService.throwException = true;
		listener.registerService(errorService);
		
		listener.received(connection, fakeCommand);
		
		assertTrue("Fake service should have been executed.", fakeCommandService.executed);
	}
	
	@Test
	/**
	 * Listener should not forward to services objects that are not FLNetworkCommands.
	 */
	public void testListenerDoesNotForwardNonCommandObjects() {
		listener.received(connection, new Object());
		assertFalse("Services should not be executed.", this.fakeCommandService.executed);
	}
	
	/** Elementary test: do not run commands if the security manager forbids it, but do so if it is allowed. */
	@Test
	public void respectVetoOfSecurityManager() {
		FakeSecurityManager fakeSecurity = new FakeSecurityManager();
		listener = new FLNetworkServerListener(fakeSecurity, updater);
		//register the fake command service
		listener.registerService(fakeCommandService);

		fakeSecurity.allow = false;
		listener.received(connection, fakeCommand);
		assertFalse("Service was executed", fakeCommandService.executed);

		fakeSecurity.allow = true;
		listener.received(connection, fakeCommand);
		assertTrue("Command was not executed", fakeCommandService.executed);
	}


	// Some more detailed tests that examine the interplay between the
	// listener, the security manager, and the command executor. The correct
	// order for receiving network commands is:
	//
	// * the listener gets a NetworkRegisterCommand
	// * executing this command sets the UUID of the corresponding connection
	// * After that, arbitrary commands (except NetworkRegisterCommands) can be
	//   sent and are processed.
	//
	// Any deviation from this procedure should not be processed; usually, they
	// are rejected by the SecurityManager.


	@Test
	public void listenerReceivesANetworkRegisterCommandAndExecutesIt() {
		connection = new FakeFLNetworkConnection();
		listener.received(connection, registerCommand);

		assertNotNull("Server did not generate a connection UUID.", connection.getClientId());
		assertEquals("Server did not set the client name.", registerCommand.getClientName(), connection.getClientName());
		assertEquals("Server did not generate the default Client role.", ClientRole.GUEST, connection.getRole());
	}

	@Test
	public void listenerReceivesANetworkRegisterCommandFromAlreadyRegisteredPlayer() {
		UUID connectionUUID = connection.getClientId();
		ClientRole expectedRole = connection.getRole();

		listener.received(connection, registerCommand);

		assertEquals("Command was executed even though client was already registered.", connectionUUID, connection.getClientId());
		assertEquals("Command was executed even though client was already registered.", expectedRole, connection.getRole());
	}

	@Test
	public void listenerReceivesACommandFromAnUnregisteredPlayer() {
		connection = new FakeFLNetworkConnection();

		listener.received(connection, fakeCommand);
		assertFalse("Service was executed.", fakeCommandService.executed);
	}

	@Test
	public void listenerReceivesAGenericCommandFromARegisteredPlayer() {
		listener.received(connection, fakeCommand);

		assertTrue("Service was not executed.", fakeCommandService.executed);
	}

	@Test
	public void listenerReceivesAClientRegisterNameCommandFromARegisteredPlayer() {
		connection = new FakeFLNetworkConnection();

		String expectedName="Patton";
		ClientNetworkRegisterCommand command = new ClientNetworkRegisterCommand(expectedName);

		listener.received(connection, command);
		assertEquals("Server did not register the player name.", expectedName, connection.getClientName());
	}
}
