/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.freelords.network.fake.FakeNetworkCommand;
import org.freelords.network.server.fake.FakeKryonetServer;
import org.junit.Before;
import org.junit.Test;

import com.esotericsoftware.kryonet.Listener;

public class FLNetworkServerTest {

	private FakeKryonetServer fakeKryonetServer = new FakeKryonetServer();
	private FLNetworkServer server = new FLNetworkServer(fakeKryonetServer);
	private FLNetworkServer unconfiguredServer = new FLNetworkServer(fakeKryonetServer);

	@Before
	public void setUp() throws Exception {
		server.bindPort(11111);
		server.registerClass(FakeNetworkCommand.class);
		server.addReceivingListener(new Listener());
		server.initNetwork();
	}

	@Test(expected = IllegalArgumentException.class)
	public void CreateServerWithNullKryonetServer() {
		server = new FLNetworkServer(null);
	}

	@Test(expected = IOException.class)
	public void startUnconfiguredServer() throws IOException {
		assertFalse("Server was running before starting it.", fakeKryonetServer.isRunning);
		unconfiguredServer.start();
		assertFalse("Kryonet Server was running.", fakeKryonetServer.isRunning);
	}

	@Test(expected = IOException.class)
	public void stopNotRunningServer() throws IOException {
		server.stop();
	}

	@Test(expected = IOException.class)
	public void configureServerWithoutAddingNetworkCommands() throws IOException {
		unconfiguredServer.bindPort(11111);
		unconfiguredServer.addReceivingListener(new Listener());
		unconfiguredServer.initNetwork();		
	}
	
	@Test
	public void configureServerCorrectly() throws IOException {
		assertTrue("Port bind is not done.", fakeKryonetServer.bindCalled);
		assertTrue("The class was not registered correctly.", server.registeredClasses.contains(FakeNetworkCommand.class));
		assertEquals("Server did not register the correct number of classes.", 1, fakeKryonetServer.kryo.registerCalled);
		assertTrue("Server did not register a listener.", fakeKryonetServer.listenerAdded);
	}

	@Test
	public void startWellConfiguredServer() throws IOException {
		assertFalse("Server was running before starting it.", fakeKryonetServer.isRunning);
		server.start();
		assertTrue("Kryonet Server was not running.", fakeKryonetServer.isRunning);
	}

	@Test(expected = IOException.class)
	public void avoidStartingServerTwice() throws IOException {
		server.start();
		server.start();
	}

	@Test
	public void stopRunningWellConfiguredServer() throws IOException {
		assertFalse("Server was running before starting it.", fakeKryonetServer.isRunning);
		server.start();
		assertTrue("Kryonet Server was not running.", fakeKryonetServer.isRunning);
		server.stop();
		assertFalse("Kryonet Server was not stopped.", fakeKryonetServer.isRunning);
	}
}
