/*
 * Copyright (C) 2011 Anton Kirillov, Andrea Paternesi, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server;

import java.util.UUID;

import com.esotericsoftware.kryonet.Connection;

import org.freelords.network.fake.FakeFLNetworkConnection;
import org.freelords.network.fake.FakeUpdate;
import org.freelords.network.server.fake.FakeKryonetServer;
import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

public class UpdaterTest {

	private static int NUM_CONNECTION = 2;

	private Updater updater;
	private FakeKryonetServer server;
	private FakeFLNetworkConnection connection;

	@Before
	public void setup() {
		server = new FakeKryonetServer();
		updater = new Updater(server);

		// register a handful of connections
		for (int i = 0; i < NUM_CONNECTION; i++) {
			connection = new FakeFLNetworkConnection();
			connection.setClientId(UUID.randomUUID());
			server.addConnection(connection);
		}
		
		// also, register an uninitialized connection (i.e., without id).
		server.addConnection(new FakeFLNetworkConnection());
	}

	/**
	 * Test count of connections which should be returned by Updater.
	 * There are 3 connections in FakeKryonetServer, but only 2 with a client id.
	 */
	@Test
	public void testListConnections() {
		int size = updater.listConnections().size();
		assertEquals("Invalid connections count", NUM_CONNECTION, size);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testConstructionwithNullServer() {
		updater = new Updater(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testBroadcastWithNullUpdate() {
		updater.broadcastUpdate(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testSendUpdateWithNullUpdate() {
		UUID clientId = connection.getClientId();
		updater.sendUpdate(clientId, null);
	}

	/**
	 * Test that update does not send to client with invalid id
	 */
	@Test(expected = IllegalArgumentException.class)
	public void testSendUpdateFailed() {
		updater.sendUpdate(UUID.randomUUID(), new FakeUpdate());
	}
	
	/**
	 * Test that connection that have clientId received broadcast message.
	 */
	@Test
	public void testBroadcast() {
		updater.broadcastUpdate(new FakeUpdate());

		Connection[] connections = server.getConnections();
		for (int i = 0; i < connections.length; i++) {
			FakeFLNetworkConnection con = (FakeFLNetworkConnection) connections[i];
			if (con.getClientId() == null) {
				assertFalse("Broadcast to uninitialized connection.", con.isCalled());
			} else {
				assertTrue("No broadcast to valid connection.", con.isCalled());
			}
		}
	}

	/**
	 * Test that a connection with a specified id receives an Update
	 */
	@Test
	public void testSendUpdate() {
		UUID clientId = connection.getClientId();
		updater.sendUpdate(clientId, new FakeUpdate());

		Connection[] connections = server.getConnections();
		for (int i = 0; i < connections.length; i++) {
			FakeFLNetworkConnection con = (FakeFLNetworkConnection) connections[i];
			if (clientId.equals(con.getClientId())) {
				assertTrue("Connection with specified id did not receive Update.", con.isCalled());
			} else {
				assertFalse("Wrong connection was called.", con.isCalled());
			}
		}
	}
}
