/*
 * Copyright (C) 2011 Andrea Paternesi, Ulf Lorenz
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server.service;

import java.util.Arrays;
import java.util.HashSet;
import java.util.UUID;

import org.freelords.network.command.ChatBroadcastCommand;
import org.freelords.network.command.ChatMulticastCommand;
import org.freelords.network.command.exception.InvalidCommandException;
import org.freelords.network.fake.FakeFLNetworkConnection;
import org.freelords.network.fake.FakeNetworkCommand;
import org.freelords.network.server.Updater;
import org.freelords.network.server.fake.FakeKryonetServer;
import org.freelords.network.server.service.fake.FakeChatMulticastCommand;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

public class ServerChatServiceTest {

	private static final String CHAT_MESSAGE = "Chat Message";
	private ServerChatService service;
	private FakeKryonetServer server;
	private Updater updater;
	private ChatBroadcastCommand commandBroadcast;
 	private ChatMulticastCommand commandMuticast;
	private FakeFLNetworkConnection client1 = new FakeFLNetworkConnection();
	private FakeFLNetworkConnection client2 = new FakeFLNetworkConnection();
	private FakeFLNetworkConnection client3 = new FakeFLNetworkConnection();
	private UUID client1Id = UUID.randomUUID();
	private UUID client2Id = UUID.randomUUID();
	private UUID client3Id = UUID.randomUUID();

	@Before
	public void setUp() {
		commandMuticast = new ChatMulticastCommand(CHAT_MESSAGE,
				new HashSet<String>(Arrays.asList(UUID.randomUUID().toString(), UUID.randomUUID().toString())));
		commandBroadcast = new ChatBroadcastCommand(CHAT_MESSAGE);

		server = new FakeKryonetServer();
		service = new ServerChatService();
		updater = new Updater(server);

		client1.setClientId(client1Id);
		client2.setClientId(client2Id);
		client3.setClientId(client3Id);
		server.addConnection(client1);
		server.addConnection(client2);
		server.addConnection(client3);
	}

	/**
	 * The ClientChatService should only accept ChatBroadcastCommand and ChatMulticastCommand.
	 */
	@Test
	public void testAcceptsNetworkRegisterCommandAndNothingElse() {
		assertTrue("ClientChatService should accept ChatBroadcastCommand.", service.accept(commandBroadcast));
		assertTrue("ClientChatService should accept ChatMulticastCommand.", service.accept(commandMuticast));
		assertFalse("ClientChatService should not accept FakeNetworkCommand.", service.accept(new FakeNetworkCommand()));
	}

	@Test
	public void inCaseOfBroadcastCommandAllClientsReceiveTheChatMessage() throws Exception {
		service.execute(commandBroadcast, client1, updater);
		assertTrue("Client 1 did not receive the chat broadcast message.", client1.isCalled());
		assertTrue("Client 2 did not receive the chat broadcast message.", client2.isCalled());
		assertTrue("Client 3 did not receive the chat broadcast message.", client3.isCalled());
	}

	@Test(expected = InvalidCommandException.class)
	public void inCaseOfMulticastCommandAllClientIdsMustNotBeNull() throws Exception {
		FakeChatMulticastCommand fakeCommandMulticast = new FakeChatMulticastCommand(CHAT_MESSAGE,
				new HashSet<String>(Arrays.asList(client2Id.toString(), null)));
		service.execute(fakeCommandMulticast, client1, updater);
	}

	@Test(expected = InvalidCommandException.class)
	public void inCaseOfMulticastCommandAllClientIdsMustBeValid() throws Exception {
		FakeChatMulticastCommand fakeCommandMulticast = new FakeChatMulticastCommand(CHAT_MESSAGE,
				new HashSet<String>(Arrays.asList(client2Id.toString(), UUID.randomUUID().toString())));
		service.execute(fakeCommandMulticast, client1, updater);
	}

	@Test
	public void inCaseOfMulticastCommandAllClientsWhoseIdAreInTheListReceiveTheChatMessage() throws Exception {
		FakeChatMulticastCommand fakeCommandMulticast = new FakeChatMulticastCommand(CHAT_MESSAGE,
				new HashSet<String>(Arrays.asList(client3Id.toString())));
		service.execute(fakeCommandMulticast, client1, updater);

		// only client 3 should get a chat message
		assertFalse("Client 1 received the chat broadcast message.", client1.isCalled());
		assertFalse("Client 2 received the chat broadcast message.", client2.isCalled());
		assertTrue("Client 3 did not receive the chat broadcast message.", client3.isCalled());
	}
}
