/*
 * Copyright (C) 2011 Ion Pana, Anton Kirillov, Ulf Lorenz
 *               2012 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.network.server.service;

import java.util.UUID;

import org.freelords.network.client.Session;
import org.freelords.network.command.ClientNetworkRegisterCommand;
import org.freelords.network.common.ClientRole;
import org.freelords.network.fake.FakeFLNetworkConnection;
import org.freelords.network.fake.FakeNetworkCommand;
import org.freelords.network.server.Updater;
import org.freelords.network.server.fake.FakeKryonetServer;
import org.freelords.network.update.ClientListUpdate;

import static org.junit.Assert.*;
import org.junit.Before;
import org.junit.Test;

public class ServerClientRegisterServiceTest {

	private static final String PLAYER_NAME = "Leroy Jenkins";
	private static final String PLAYER_NAME2 = "Leroy Jenkins2";
	private ServerClientRegisterService service;
	private FakeFLNetworkConnection connection;
	private ClientNetworkRegisterCommand command;
	private Updater updater;
	FakeKryonetServer server;

	@Before
	public void setUp() {
		connection = new FakeFLNetworkConnection();
		command = new ClientNetworkRegisterCommand(PLAYER_NAME);
		server = new FakeKryonetServer();
		server.addConnection(connection);
		updater = new Updater(server);
		service = new ServerClientRegisterService();
	}

	@Test
	/**
	 * The ClientNetworkRegisterService should only accept ClientNetworkRegisterCommand.
	 */
	public void testAcceptsNetworkRegisterCommandAndNothingElse() {
		assertTrue("ClientNetworkRegisterService should accept ClientNetworkRegisterCommand.", service.accept(command));
		assertFalse("ClientNetworkRegisterService should not accept FakeNetworkCommand.",
				service.accept(new FakeNetworkCommand()));
	}

	@Test
	/**
	 * Client Id and Role should be initialized to GUEST and a Unique identifier. 
	 */
	public void testReceiveCommandFillsRoleAndClientIdIfNull() throws Exception {
		//call the service
		service.execute(command, connection, updater);

		assertNotNull("Client id should not be null.", connection.getClientId());
		assertEquals("Client role was not initialized to default.", ClientRole.GUEST, connection.getRole());
		assertEquals("Client name was not set properly.", command.getClientName(), connection.getClientName());
	}

	/**
	 * 2 register commands received on the same connection should throw an exception.
	 */
	@Test(expected = IllegalStateException.class)	
	public void testReceiveCommandAfterInitialization() throws Exception {
		//call the service
		service.execute(command, connection, updater);

		//second call should throw an exception
		service.execute(command, connection, null);
	}

	/**
	 * Test that new registered client receives update 
	 */
	@Test
	public void clientReceivesRegistrationUpdate() throws Exception {
		FakeFLNetworkConnection sender = new FakeFLNetworkConnection();
		server.addConnection(sender);
		service.execute(command, sender, updater);
		assertTrue("Sender did not receive acknowledgement about his ID", sender.isCalled());
	}

	/**
	 * Test that all connection of all connected users are called. 
	 * All connected users should receive instance of ClientListUpdate with valid clients ids and names
	 */
	@Test
	public void allClientsReceiveListUpdate() throws Exception {
		// set up two connections: an established one (recipient), and a freshly registered one
		FakeFLNetworkConnection sender = new FakeFLNetworkConnection();

		FakeFLNetworkConnection recipient = new FakeFLNetworkConnection();
		recipient.setClientId(UUID.randomUUID());
		recipient.setClientName(PLAYER_NAME2);

		server.addConnection(sender);
		server.addConnection(recipient);

		// now register the fresh connection
		service.execute(command, sender, updater);

		assertTrue("Every connected user should receive update",
				sender.isCalled() && recipient.isCalled());
		assertTrue("Connected users should receive instance of ClientListUpdate",
				recipient.getReceivedObject() instanceof ClientListUpdate);

		// check that the client list update contains now both registered players.
		boolean gotSenderInList = false;
		boolean gotOthersInList = false;

		ClientListUpdate update = (ClientListUpdate) recipient.getReceivedObject();
		Session clientSession=new Session();
		update.updateSession(clientSession);		
		
		for (UUID id : clientSession.listClientIds()) {
			String name=clientSession.getClientName(id);
			if (recipient.getClientId().equals(id)) {
				gotOthersInList = name.equals(PLAYER_NAME2);
			}
			if (PLAYER_NAME.equals(name)) {
				gotSenderInList = true;
			}
		}
		
		assertTrue("ClientListUpdate does not contains valid clients id and name", gotOthersInList && gotSenderInList);
	}
}
