/*
 * Copyright (C) 2011 Ulf Lorenz,Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.resourcemanager;


import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Arrays;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;


/** Tests the Repository class.
  *
  * You should be able to set valid directories or zip/jar files as resources
  * and access files below these root paths.
  *
  */

public class DataRepositoryTest {

	/** Check that the class throws an exception if fed invalid directory names. */
	@Test(expected = InvalidFileException.class)
	public void repositoryIsCreatedWithInvalidFileResource() throws InvalidFileException {
		ResourceRepository rep = new DataRepository(Arrays.asList("anInvalidFileName"));
		rep.connectToResources();
	}

	/** Check that the class throws an exception if fed an invalid zip file. */
	@Test(expected = InvalidFileException.class)
	public void repositoryIsCreatedWithInvalidZipResource() throws InvalidFileException {
		ResourceRepository rep = new DataRepository(Arrays.asList("anInvalidFile.zip"));
		rep.connectToResources();
	}

	/** Check that the class throws an exception if fed invalid jar files. */
	@Test(expected = InvalidFileException.class)
	public void repositoryIsCreatedWithInvalidJarResource() throws InvalidFileException {
		ResourceRepository rep = new DataRepository(Arrays.asList("anInvalidFile.jar"));
		rep.connectToResources();
	}


	//---------------------------------------------------------------


	/** Make sure that the repository requires connectToResources(). */
	@Test(expected = IllegalStateException.class)
	public void accessValidFileWithoutInitialisation() throws Exception {
		String rootpath = this.getClass().getResource("data").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));

		rep.getStream(ResourcePackage.TEST, "test", ResourceType.TEXT);
	}

	/** Make sure that accessing an incorrect resource gives an error. */
	@Test(expected = FileNotFoundException.class)
	public void accessInvalidFile() throws Exception {
		String rootpath = this.getClass().getResource("data").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));
		rep.connectToResources();
		rep.getStream(ResourcePackage.TEST, "thisFileShouldNotExist", ResourceType.TEXT);
	}


	//---------------------------------------------------------------


	/** Try to open a valid directory and access some file with content there. */
	@Test
	public void accessValidFileRepository() throws Exception {
		// create a file repository under the directory "data/"
		String rootpath = this.getClass().getResource("data").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));
		rep.connectToResources();

		// get an input stream and make sure it is correct.
		InputStream input = rep.getStream(ResourcePackage.TEST, "test", ResourceType.TEXT);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("Input stream gave wrong string", readin.readLine().equals("text"));
		input.close();
	}

	/** Try to open a valid zip file and access some file with content there. */
	@Test
	public void accessValidZipRepository() throws Exception {
		// create a zip repository from the file "data.zip"
		String rootpath = this.getClass().getResource("data.zip").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));
		rep.connectToResources();

		// get an input stream and make sure it is correct.
		InputStream input = rep.getStream(ResourcePackage.TEST, "test", ResourceType.TEXT);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("Input stream gave wrong string", readin.readLine().equals("text"));
		input.close();
	}

	/** Try to open a valid jar file and access some file with content there. */
	@Test
	public void accessValidJarRepository() throws Exception {
		// create a jar repository from the file "data.jar"
		String rootpath = this.getClass().getResource("data.jar").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));
		rep.connectToResources();

		// get an input stream and make sure it is correct.
		InputStream input = rep.getStream(ResourcePackage.TEST, "test", ResourceType.TEXT);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("Input stream gave wrong string", readin.readLine().equals("text"));
		input.close();
	}


	//---------------------------------------------------------------


	/** Test that all variants of the enum Package can be opened properly.
	  *
	  * If this test fails, then probably a new package type was added or the
	  * directory layout was changed. Consider a failure of this function a
	  * hint that you should stop and think about the consequences (e.g., by
	  * changing the directory types in the actual data repositories).
	  */
	// added ignore, since this test failed on windows when run from ant file
	@Ignore
	@Test
	public void testThroughAllPackageTypes() throws Exception {
		String rootpath = this.getClass().getResource("data").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath));
		rep.connectToResources();

		for (ResourcePackage pack : ResourcePackage.values()) {
			rep.getStream(pack, "exists", ResourceType.TEXT).close();
		}
	}


	//---------------------------------------------------------------


	/** Tests combination of multiple repositories, so you see the adjoint of them. */
	@Test
	public void combineMultipleRepositories() throws Exception {
		// Set up two directory repositories
		String rootpath  = this.getClass().getResource("data").getFile();
		String rootpath2 = this.getClass().getResource("data2").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath, rootpath2));
		rep.connectToResources();

		// One of them has only item2.txt, the other one item3.txt.
		rep.getStream(ResourcePackage.TEST, "item2", ResourceType.TEXT).close();
		rep.getStream(ResourcePackage.TEST, "item3", ResourceType.TEXT).close();
	}

	/** Test that higher priority repositories always shadow lower priority ones. */
	@Test
	public void testShadowingOfRepositories() throws Exception {
		// Set up two directory repositories
		String rootpath  = this.getClass().getResource("data").getFile();
		String rootpath2 = this.getClass().getResource("data2").getFile();
		ResourceRepository rep = new DataRepository(Arrays.asList(rootpath, rootpath2));
		rep.connectToResources();

		// Both repositories have "item1.txt", but "data" has a higher priority.
		InputStream input = rep.getStream(ResourcePackage.TEST, "item1", ResourceType.TEXT);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("Incorrect order of shadowing.", readin.readLine().equals("data"));
		input.close();
	}
}
