/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.helpers.DefaultValidationEventHandler;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import org.freelords.resourcemanager.DataRepository;
import org.freelords.resourcemanager.ResourceRepository;
import org.freelords.sound.command.SoundCommand;
import org.freelords.sound.fake.FakeClip;
import org.freelords.sound.fake.FakeMixer;
import org.freelords.sound.fake.MuteableFakeClip;
import org.freelords.sound.fake.VolumeFakeClip;

/** Tests the audio manager class. */

public class AudioManagerTest {
	private BlockingQueue<SoundCommand> queue;
	private AudioManager manager;

	private FakeMixer mixer;
	private InputStream soundStream;
	private InputStream anotherStream;

	private ScenarioEventSounds playlist;

	/** Setup the tests. */
	@Before
	public void setup() throws Exception {
		// set up the audio manager
		mixer = new FakeMixer(10);
		queue = new ArrayBlockingQueue<SoundCommand>(20);
		manager = new AudioManager(mixer, queue);

		File musicFile = new File(this.getClass().getResource("full/test/test1.ogg").getFile());
		soundStream = new BufferedInputStream(new FileInputStream(musicFile));
		anotherStream = new BufferedInputStream(new FileInputStream(musicFile));

		// load some ScenarioEventSounds instance from a file and set it up
		JAXBContext jc = JAXBContext.newInstance(ScenarioEventSounds.class, SoundEvent.class);
		Unmarshaller unmarshaller = jc.createUnmarshaller();
		unmarshaller.setEventHandler(new DefaultValidationEventHandler());
		File xmlFile = new File(this.getClass().getResource("managertest.xml").getFile());

		playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();
		playlist.setResourceRepository(rep);
	}

	private void flushQueue() {
		while (!queue.isEmpty()) {
			queue.poll().run();
		}
	}

	/** Check that playing a simple sound works. */
	@Test
	public void testSoundPlaying() {
		String ref = manager.playSound(soundStream);
		flushQueue();

		Assert.assertTrue("audio player was not started.", mixer.getClip(0).isRunning());
		Assert.assertNotNull("playSound() returned null reference.", ref);
	}

	/** Test that the looping interface works. */
	@Test
	public void testSoundLooping() throws Exception {
		// play a looping sound with non-random 100 ms gap between repeats.
		manager.playSound(soundStream, true, 100, false);
		flushQueue();
		final FakeClip clip = mixer.getClip(0);

		// finish the sound in a separate stream, and make sure that it takes
		// some time before it runs again.
		clip.setFramePosition(clip.getFrameLength());
		Thread t = new Thread() {

			@Override
			public void run() {
				clip.stop();
			}
		};
		t.start();

		Thread.sleep(10);
		Assert.assertFalse("Clip is still running despite having been terminated.", clip.isRunning());
		
		Thread.sleep(100);
		Assert.assertTrue("Clip is not looping", clip.isRunning());
	}

	/** Test that multiple sounds can be played in parallel. */
	@Test
	public void testMultiplePlaying() {
		manager.playSound(soundStream);
		manager.playSound(anotherStream);
		flushQueue();

		Assert.assertTrue("First clip failed to run.", mixer.getClip(0).isRunning());
		Assert.assertTrue("Second clip failed to run.", mixer.getClip(1).isRunning());
	}

	/** Test that we get an exception for invalid streams. */
	@Test(expected = IllegalArgumentException.class)
	public void throwExceptionOnInvalidStream() throws Exception {
		File musicFile = new File(this.getClass().getResource("full/test/test.ogg").getFile());
		InputStream invalidStream = new BufferedInputStream(new FileInputStream(musicFile));

		manager.playSound(invalidStream);
	}

	/** Test that negative steppings are forbidden. */
	@Test(expected = IllegalArgumentException.class)
	public void throwExceptionOnNegativeStepping() {
		manager.playSound(soundStream, true, -100, false);
	}

	/** Test that otherwise, the player silently fails; especially for missing lines. */
	@Test
	public void silentFailOnMissingLines() {
		mixer = new FakeMixer(0);
		manager = new AudioManager(mixer, queue);

		String ref = manager.playSound(soundStream);

		Assert.assertTrue("AudioManager returned reference although playing failed.", ref == null);
	}


	/** The manager should throw an exception if no event mapping has been set. */
	@Test(expected = IllegalStateException.class)
	public void testExceptionOnMissingEventMapping() {
		manager.playEvent(SoundEvent.BACKGROUND);
	}

	/** Test that a sound event mapping can be set and sounds can be played. */
	@Test
	public void testEventSoundPlaying () {
		manager.setEventSounds(playlist);
		manager.playEvent(SoundEvent.BACKGROUND);
		flushQueue();

		Assert.assertTrue("Event sound is not running", mixer.getClip(0).isRunning());
	}

	/** Make sure the player fails silently on various conditions. */
	@Test
	public void testSilentFailingOfEventPlaying() {
		mixer = new FakeMixer(0);
		manager = new AudioManager(mixer, queue);
		manager.setEventSounds(playlist);

		// events for which no sound is set
		manager.playEvent(SoundEvent.DEFEAT);

		// events for which the input cannot be converted into an AudioStream
		manager.playEvent(SoundEvent.VICTORY);

		// no lines are available.
		manager.playEvent(SoundEvent.BACKGROUND);
	}

	/** Test that an event sound is interrupted if another starts to play. */
	@Test
	public void eventSoundsAreInterruptedProperly() {
		// first, the player starts playing an event, using the first clip.
		manager.setEventSounds(playlist);
		manager.playEvent(SoundEvent.BACKGROUND);
		flushQueue();

		Assert.assertTrue("Event sound is not running.", mixer.getClip(0).isRunning());
		Assert.assertFalse("Second clip is running although only one was started.", mixer.getClip(1).isRunning());

		// then it plays another event, killing the first clip and using the second.
		manager.playEvent(SoundEvent.NEW_HERO);
		flushQueue();

		Assert.assertFalse("Old event sound is still running.", mixer.getClip(0).isRunning());
		Assert.assertTrue("No new event sound was started.", mixer.getClip(1).isRunning());
	}

	/** Make sure that the event player loops by default. */
	@Test
	public void testEventManagerLooping() {
		manager.setEventSounds(playlist);
		manager.playEvent(SoundEvent.BACKGROUND);
		flushQueue();

		Assert.assertTrue("Event sound is not running", mixer.getClip(0).isRunning());

		// this pretends that the clip has finished playing
		// the manager should start a new player, which uses up a new clip.
		mixer.getClip(0).quitPlaying();
		flushQueue();

		Assert.assertFalse("Old clip is still running", mixer.getClip(0).isRunning());
		Assert.assertTrue("No new clip has been started for event sound looping.", mixer.getClip(1).isRunning());
	}

	/** Try to stop an event. */
	@Test
	public void removePlayingEvent() {
		manager.setEventSounds(playlist);

		manager.playEvent(SoundEvent.BACKGROUND);
		flushQueue();
		Assert.assertTrue("Event sound has not been started.", mixer.getClip(0).isRunning());

		manager.removeEvent();
		flushQueue();
		Assert.assertTrue("Event sound has not been stopped properly.", mixer.getClip(0).isClosed());
	}

	/** Make sure that the manager does not complain on forgotten tokens. */
	@Test
	public void removeNotPlayingSound() {
		manager.removeSound(UUID.randomUUID().toString());
	}

	/** Test that the manager can kill playing sounds. */
	@Test
	public void removePlayingSound() {
		String ref = manager.playSound(soundStream);
		flushQueue();
		Assert.assertTrue("Sound has not been started.", mixer.getClip(0).isRunning());

		manager.removeSound(ref);
		flushQueue();
		Assert.assertTrue("Sound has not been stopped properly.", mixer.getClip(0).isClosed());
	}

	/** When calling finish, playing events and sounds are killed. */
	@Test
	public void killAllSound() {
		manager.setEventSounds(playlist);
		manager.playEvent(SoundEvent.BACKGROUND);
		manager.playSound(soundStream);
		manager.finish();
		flushQueue();

		Assert.assertTrue("Event sound has not been stopped properly.", mixer.getClip(0).isClosed());
		Assert.assertTrue("Sound has not been stopped properly.", mixer.getClip(1).isClosed());
	}

	/** Replaces the mixer by something with MuteableFakeClips. */
	private void setupMutableMixer() {
		List<FakeClip> l = new ArrayList<FakeClip>();
		l.add(new MuteableFakeClip());
		l.add(new MuteableFakeClip());
		mixer = new FakeMixer(l);
		manager = new AudioManager(mixer, queue);
		manager.setEventSounds(playlist);
	}

	/** Mute all the sound pieces. */
	@Test
	public void muteAllSound() {
		setupMutableMixer();

		manager.playSound(soundStream);
		flushQueue();

		manager.stopSound();
		flushQueue();

		Assert.assertTrue("Clip has not been muted.", ((MuteableFakeClip)mixer.getClip(0)).muting.getValue());
	}

	/** Mute everything (sound plus event). */
	@Test
	public void muteEverything() {
		setupMutableMixer();

		manager.playEvent(SoundEvent.BACKGROUND);
		manager.playSound(soundStream);
		manager.stopSystem();
		flushQueue();

		Assert.assertTrue("Clip has not been muted.", ((MuteableFakeClip)mixer.getClip(0)).muting.getValue());
		Assert.assertTrue("Clip has not been muted.", ((MuteableFakeClip)mixer.getClip(1)).muting.getValue());
	}

	/** While everything is muted, new players always start silently. */
	@Test
	public void startPlayersMuted() {
		setupMutableMixer();

		manager.stopSystem();
		manager.playEvent(SoundEvent.BACKGROUND);
		manager.playSound(soundStream);
		flushQueue();

		Assert.assertTrue("Clip was started unmuted during muted session.", ((MuteableFakeClip)mixer.getClip(0)).muting.getValue());
		Assert.assertTrue("Clip was started unmuted during muted session.", ((MuteableFakeClip)mixer.getClip(1)).muting.getValue());
	}

	/** Test that unmuting works. */
	@Test
	public void testResuming() {
		setupMutableMixer();

		manager.playEvent(SoundEvent.BACKGROUND);
		manager.playSound(soundStream);
		manager.stopSystem();
		manager.resumeSystem();
		flushQueue();

		Assert.assertFalse("Clip was not properly resumed.", ((MuteableFakeClip)mixer.getClip(0)).muting.getValue());
		Assert.assertFalse("Clip was not properly resumed.", ((MuteableFakeClip)mixer.getClip(1)).muting.getValue());
	}

	private void setupVolumeMixer() {
		List<FakeClip> l = new ArrayList<FakeClip>();
		l.add(new VolumeFakeClip());
		l.add(new VolumeFakeClip());
		mixer = new FakeMixer(l);
		manager = new AudioManager(mixer, queue);
		manager.setEventSounds(playlist);
	}

	/** Test that volume of events can be changed. */
	@Test
	public void testEventVolumeChanges() {
		setupVolumeMixer();

		manager.playEvent(SoundEvent.BACKGROUND);
		manager.setEventVolume(50);
		flushQueue();

		VolumeFakeClip vfc = (VolumeFakeClip)mixer.getClip(0);
		Assert.assertEquals("Amplitude level is not half of the maximum",
				Math.pow(10, vfc.volume.getMaximum()/20) / 2,
				Math.pow(10, vfc.volume.getValue()/20),
				0.1);
	}

	/** Test that the volume of sounds can be changed. */
	@Test
	public void testSoundVolumeChanges() {
		setupVolumeMixer();

		manager.playSound(soundStream);
		manager.setSoundVolume(50);
		flushQueue();

		VolumeFakeClip vfc = (VolumeFakeClip)mixer.getClip(0);
		Assert.assertEquals("Amplitude level is not half of the maximum",
				Math.pow(10, vfc.volume.getMaximum()/20) / 2,
				Math.pow(10, vfc.volume.getValue()/20),
				0.1);
	}

	/** Test that volume changes remain in effect for players started after the setting. */
	@Test
	public void testVolumeChangePersistency() {
		setupVolumeMixer();

		manager.setEventVolume(50);
		manager.setSoundVolume(50);
		manager.playEvent(SoundEvent.BACKGROUND);
		manager.playSound(soundStream);
		flushQueue();

		VolumeFakeClip vfc = (VolumeFakeClip)mixer.getClip(0);
		Assert.assertEquals("Amplitude level is retained for freshly started players.",
				Math.pow(10, vfc.volume.getMaximum()/20) / 2,
				Math.pow(10, vfc.volume.getValue()/20), 0.1);

		vfc = (VolumeFakeClip)mixer.getClip(1);
		Assert.assertEquals("Amplitude is not retained for freshly started players.", 
				Math.pow(10, vfc.volume.getMaximum()/20) / 2,
				Math.pow(10, vfc.volume.getValue()/20), 0.1);
	}
}
