/*
 * Copyright (C) 2011 Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.BooleanControl;
import javax.sound.sampled.FloatControl;

import org.freelords.sound.fake.FakeClip;
import org.freelords.sound.fake.MuteableFakeClip;
import org.freelords.sound.fake.VolumeFakeClip;

import org.junit.Before;
import org.junit.Test;
import org.junit.Assert;

/** Tests the AudioPlayer for correct behaviour. */
public class ClipPlayerTest {

	/** The input file stream. */
	private InputStream fileIn;
	/** The input audio stream. */
	private AudioInputStream audioIn;
	private AudioPlayer player;
	private FakeClip myclip = new FakeClip();
	private MuteableFakeClip myclipmutable = new MuteableFakeClip();
	private VolumeFakeClip myclipvolume = new VolumeFakeClip();
	private TestAudioListener listener1 = new TestAudioListener();
	private TestAudioListener listener2 = new TestAudioListener();

	/** Set up the standard player and such. */
	@Before
	public void setupStuff() throws Exception {
		File musicFile = new File(this.getClass().getResource("full/test/test1.ogg").getFile());
		fileIn = new BufferedInputStream(new FileInputStream(musicFile));
		audioIn = AudioSystem.getAudioInputStream(fileIn);
		player = new ClipPlayer(myclip, audioIn);
	}

	/** Test that a player, if created properly, plays some sound using the
	 *  supplied Clip. */
	@Test(expected = IOException.class)
	public void playerPlaysSounds() throws Exception {
		// although this would happen in practice, we do not put the player in
		// a separate thread here. There is no unwanted time delay here,
		// because the FakeClip does not actually do anything.
		player.start();

		Assert.assertEquals("Stream was not properly supplied to the clip.", audioIn, myclip.getAudioInputStream());
		Assert.assertTrue("Clip has not been started.", myclip.isRunning());

		// manually force the clip to stop, and check that it was cleaned up properly
		myclip.quitPlaying();
		Assert.assertFalse("Clip has not been stopped.", myclip.isRunning());
		Assert.assertTrue("Clip has not been cleaned up.", myclip.isClosed());

		// if the input stream has been closed properly, this will throw an IOException
		fileIn.available();
	}

	/** Tests that playback can be stopped if we wish so. */
	@Test
	public void tryToStopPlayer() throws Exception {
		// start "playback"
		player.start();

		// Now stop playback, and check that the player has cleaned the sound
		// data up properly.
		player.stop();

		Assert.assertFalse("Clip has not been stopped.", myclip.isRunning());
		Assert.assertTrue("Clip has not been cleaned up.", myclip.isClosed());
	}

	/** Test that the playback can be muted and unmuted. */
	@Test
	public void tryToMutePlayer() throws Exception {
		player = new ClipPlayer(myclipmutable, audioIn);
		BooleanControl muteControl = (BooleanControl) myclipmutable.getControl(BooleanControl.Type.MUTE);

		// start "playback"
		player.start();

		// Now mute the player. The clip should then be muted, but playback should continue.
		player.mute();

		Assert.assertTrue("Clip was stopped instead of being muted.", myclipmutable.isRunning());
		Assert.assertTrue("Clip was not muted.", muteControl.getValue());

		// Now unmute the player again.
		player.resume();

		Assert.assertFalse("Clip was not unmuted.", muteControl.getValue());
	}

	/** Test that muting throws an exception if the clip does not support it. */
	@Test(expected = UnsupportedOperationException.class)
	public void watchMutingFail() throws Exception {
		// Run the player and try to mute it. This should raise an exception, because FakeClip does not
		// have any controls.
		player.start();
		player.mute();
	}

	/** Test that unmuting throws an exception if the clip does not support it. */
	@Test(expected = UnsupportedOperationException.class)
	public void watchUnmutingFail() throws Exception {
		// Run the player and try to mute it. This should raise an exception, because FakeClip does not
		// have any controls.
		player.start();
		player.resume();
	}

	/** Test that changing the volume works. */
	@Test
	public void testVolumeChanging() throws Exception {
		player = new ClipPlayer(myclipvolume, audioIn);
		FloatControl volumeControl = (FloatControl) myclipvolume.getControl(FloatControl.Type.MASTER_GAIN);

		// the player allows volume settings between 0 and 100. Make sure this actually works.
		// allow for some rounding error.
		player.setVolume(0);
		Assert.assertEquals("Volume is not set to the minimum.",
				volumeControl.getMinimum(), volumeControl.getValue(), 0.1);

		player.setVolume(100);
		Assert.assertEquals("Volume is not set to the maximum.",
				volumeControl.getMaximum(), volumeControl.getValue(), 0.1);

		player.setVolume(50);
		Assert.assertEquals("Volume is not set to half the maximum amplitude.",
				Math.pow(10, volumeControl.getMaximum()/20) / 2,
				Math.pow(10, volumeControl.getValue()/20),
				0.1);
	}

	/** Test that too large volumes are rejected. */
	@Test(expected = IllegalArgumentException.class)
	public void testTooLargeVolume() throws Exception {
		player = new ClipPlayer(myclipvolume, audioIn);
		player.setVolume(101);
	}

	/** Test that too small volumes are rejected. */
	@Test(expected = IllegalArgumentException.class)
	public void testTooSmallVolume() throws Exception {
		player = new ClipPlayer(myclipvolume, audioIn);
		player.setVolume(-1);
	}

	/** Volume setting throws an exception if the control is not available. */
	@Test(expected = UnsupportedOperationException.class)
	public void testVolumeFailing() throws Exception {
		player.setVolume(0);
	}

	/** Test registering and deregistering of listeners to the player. */
	@Test
	public void testListening() throws Exception {
		player.addAudioListener(listener1);
		player.addAudioListener(listener2);
		player.start();

		// now finish playback and check that the listener was actually called
		myclip.quitPlaying();

		Assert.assertTrue("Listener was not notified of stopping.", listener1.hasBeenCalled());
		Assert.assertTrue("Listener was not notified of stopping.", listener2.hasBeenCalled());
	}

	/** Test that listeners are also called when the playback is forced to finish. */
	@Test
	public void testListeningOnQuitting() throws Exception {
		player.addAudioListener(listener1);
		player.addAudioListener(listener2);
		player.start();

		// now finish playback and check that the listener was actually called
		player.stop();

		Assert.assertTrue("Listener was not notified of stopping.", listener1.hasBeenCalled());
		Assert.assertTrue("Listener was not notified of stopping.", listener2.hasBeenCalled());
	}

	/** Test that listeners can be unregistered and are not called then. */
	@Test
	public void testUnregisteringOfListeners() throws Exception {
		TestAudioListener listener = new TestAudioListener();

		// the player should silently fail if the listener was never registered
		player.removeAudioListener(listener);

		// start the player, register and unregister the listener
		player.addAudioListener(listener);
		player.start();
		player.removeAudioListener(listener);

		// now finish playback and make sure the listener was ignored.
		player.stop();

		Assert.assertFalse("Listener was notified despite being unregistered.", listener.hasBeenCalled());
	}

	/** Test that the player fails on invalid use. */
	@Test(expected = IllegalStateException.class)
	public void testIllegalHandling() throws Exception {
		// Run the player, stop it and run again. This must fail because the
		// stopping has released all resources.
		player.start();
		player.stop();
		player.start();
	}

	/** Test for thread-safety. */
	@Test
	public void testThreading() throws Exception {
		// We test by first making all clips block when you try to open them
		// (this is usually done in a separate thread because it can take some
		// time to open a sound file).
		myclip = new FakeClip(true);
		player = new ClipPlayer(myclip, audioIn);

		// Then, we start the player in a separate thread. The player
		// diligently opens the clip, the clip blocks, and this thread falls
		// asleep.
		Thread playerThread = new Thread(new Runnable() {
				@Override
				public void run() {
					player.start();
				}
		});
		playerThread.start();
		playerThread.join(10);

		// Since we use ugly timing together with threads, we'd better make
		// sure in advance that the thread is running before we go on.
		Assert.assertEquals("player thread is not running. Epic fail for the test.\n",
				Thread.State.WAITING, playerThread.getState());

		// Now for each function that might create problems with threading, we
		// test that this function also blocks. We do this by running it in a
		// separate thread, waiting, say, 100 msec for this thread to finish,
		// and checking if the thread has finished or not.

		// adding a listener
		Thread t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.addAudioListener(listener1);
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("addAudioListener() is not synchronized properly.", Thread.State.BLOCKED, t.getState());

		// removing a listener
		t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.removeAudioListener(listener1);
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("removeAudioListener() is not properly synchronized.", Thread.State.BLOCKED, t.getState());

		// stopping
		t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.stop();
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("stop() is not properly synchronized.", Thread.State.BLOCKED, t.getState());

		// muting
		t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.mute();
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("mute() is not properly synchronized.", Thread.State.BLOCKED, t.getState());

		// resuming
		t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.resume();
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("resume() is not properly synchronized.", Thread.State.BLOCKED, t.getState());

		// setVolume
		t = new Thread(new Runnable() {

			@Override
			public void run() {
				player.setVolume(0);
			}
		});
		t.start();
		t.join(10);

		Assert.assertEquals("setVolume() is not properly synchronized.", Thread.State.BLOCKED, t.getState());
	}

	/** Tests that looping works. */
	@Test
	public void testLooping() throws Exception {
		// Change the player to one that loops with a non-random 100 ms break between repeats.
		player = new ClipPlayer(myclip, audioIn, true, 100, false);

		// start playing
		player.start();

		// now simulate in a separate thread that we have reached the end of
		// the piece. We then wait for >100 ms, and look if the line is running
		// again. If so, looping should work as expected.
		myclip.setFramePosition(myclip.getFrameLength());
		Thread t = new Thread(new Runnable() {

			@Override
			public void run() {
				myclip.stop();
			}
		});
		t.start();

		// after 10 ms, everything should have happily unwound itself, and the
		// clip is asleep.
		Thread.sleep(10);
		Assert.assertFalse("Clip is still running", myclip.isRunning());
		Assert.assertFalse("Clip was cleaned up although it should be looped.", myclip.isClosed());

		// after another >100 ms, the player should be playing again from the start.
		Thread.sleep(140);
		Assert.assertTrue("Clip has not been restarted.", myclip.isRunning());
		Assert.assertFalse("Clip was cleaned up although it should be looped.", myclip.isClosed());
		Assert.assertEquals("Clip was not started from the beginning.", 0, myclip.getFramePosition());

		// The looping can be interrupted by explicitely killing the player.
		player.stop();

		Assert.assertTrue("Clip was not cleaned up.", myclip.isClosed());
	}
}
