/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.helpers.DefaultValidationEventHandler;

import org.junit.Before;
import org.junit.Test;
import org.junit.Assert;

import org.freelords.reflection.ReflectionHelper;
import org.freelords.resourcemanager.DataRepository;
import org.freelords.resourcemanager.ResourceRepository;
import org.freelords.xmlmanager.utils.XPathHelper;

import static org.junit.Assert.*;


/** Tests the ScenarioEventSounds class.
  *
  * You should be able to create an instance by loading it from a xml file. It
  * can be serialized/deserialized for sending it over a network, it can
  * validate that all the files are actually there, and return entries.
  *
  */

public class ScenarioEventSoundsTest {

	private Unmarshaller unmarshaller;
	
	@Before
	public void setupTests() throws Exception {
		JAXBContext jc  = JAXBContext.newInstance(ScenarioEventSounds.class, SoundEvent.class);
		unmarshaller = jc.createUnmarshaller();
		jc.createMarshaller();
		unmarshaller.setEventHandler(new DefaultValidationEventHandler());
	}

	/** Test that an instance can be constructed from XML and marshalled again. */
	//TODO this is way too messy. Either provide a public interface to check what was read from
	//the XML or skip testing for JAXB marshalling. 
	//Another solution would be to make a generic way of handling the JAXB testing. There is task for this.
	//Discuss in chat.
	@Test
	public void createFromXMLFile() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		XPathHelper helper = new XPathHelper(xmlFile);
		
		Map<SoundEvent, List<SoundMapping>> mapping = ReflectionHelper.getFieldValue(playlist, "mapping");
		
		for (Entry<SoundEvent, List<SoundMapping>> entry : mapping.entrySet()) {
			String key = entry.getKey().toString();
			List<SoundMapping> soundMappings = entry.getValue();
			
			int nodeCount = helper.evaluateXPathAsInteger("count(//entry[@key = '" + key + "']/value)");
			
			assertEquals("Values count should match.", nodeCount, soundMappings.size());
			
			for (SoundMapping soundMapping : soundMappings) {
				boolean mappingFound = false;
				for (int i = 1 ; i <= nodeCount ; i++) {
					String xPathDirectoryValue = helper
													.evaluateXPathAsString("//entry[@key = '" + key + "']/value[" + i + "]/package/text()")
													.toLowerCase();
					String xPathFileValue = helper
											.evaluateXPathAsString("//entry[@key = '" + key + "']/value[" + i + "]/file/text()");
					
					if (xPathDirectoryValue.equals(soundMapping.getDirectory().getName()) 
							&& xPathFileValue.equals(soundMapping.getFile())) {
						//found the mapping
						mappingFound = true;
					}
				}
				
				assertTrue("Sound mapping should be in the actual file.", mappingFound);
			}
		}
	}

	/** When passing a ResourceRepository that lacks some files, we should get an exception. */
	@Test(expected = FileNotFoundException.class)
	public void passAnInvalidResourceRepository() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String emptyDirectory = this.getClass().getResource("empty").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(emptyDirectory));
		rep.connectToResources();

		playlist.setResourceRepository(rep);
	}

	/** When passing a ResourceRepository with all files, we should be fine. */
	@Test
	public void passValidResourceRepository() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();

		playlist.setResourceRepository(rep);
	}

	/** If you query some non-defined key, you get an error. */
	@Test(expected = IllegalArgumentException.class)
	public void requestInvalidKey() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();

		playlist.setResourceRepository(rep);
		playlist.getEntry(SoundEvent.DEFEAT);
	}

	/** If you query a key with one entry, you get a well-defined answer. */
	@Test
	public void requestValidSingleEntry() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();

		// VICTORY maps to a single file test.ogg with content "test"
		playlist.setResourceRepository(rep);
		InputStream input = playlist.getEntry(SoundEvent.VICTORY);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("Incorrect file mapping.", readin.readLine().equals("test"));
		input.close();
	}

	/** If you query a key with multiple entries, you get a random result. */
	@Test
	public void requestValidMultipleEntry() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();

		// BACKGROUND maps to a two files: test.ogg with content "test", and test2.ogg with "test2"
		playlist.setResourceRepository(rep);

		boolean foundtest = false;
		boolean foundtest2= false;
		// this can break once in 2^100 cases (i.e., very rarely)
		for (int i = 0; i < 100; i++) {
			InputStream input = playlist.getEntry(SoundEvent.BACKGROUND);
			BufferedReader readin = new BufferedReader(new InputStreamReader(input));
			String line = readin.readLine();

			if (line.equals("test")) {
				foundtest = true;
			}
			else if (line.equals("test2")) {
				foundtest2 = true;
			}
		}

		Assert.assertTrue("Did not retrieve both entries in event map.", foundtest && foundtest2);
	}

	/** Try to send the sound map through a pipe unharmed (tests serialization). */
	@Test
	public void testSerialization() throws Exception {
		File xmlFile = new File(this.getClass().getResource("eventsounds.xml").getFile());
		ScenarioEventSounds playlist = (ScenarioEventSounds) unmarshaller.unmarshal(xmlFile);

		File tmpFile = File.createTempFile("test", "dat");
		tmpFile.deleteOnExit();

		FileOutputStream fout = new FileOutputStream(tmpFile);
		ObjectOutputStream objOut = new ObjectOutputStream(fout);
		objOut.writeObject(playlist);
		objOut.close();

		FileInputStream fin = new FileInputStream(tmpFile);
		ObjectInputStream objIn = new ObjectInputStream(fin);
		ScenarioEventSounds newList = (ScenarioEventSounds) objIn.readObject();
		objIn.close();

		// Finally, test that some of the previous test is possible again.
		String dir = this.getClass().getResource("full").getFile();
		ResourceRepository rep = new DataRepository(Collections.singletonList(dir));
		rep.connectToResources();

		// VICTORY maps to a single file test.ogg with content "test"
		newList.setResourceRepository(rep);
		InputStream input = newList.getEntry(SoundEvent.VICTORY);
		BufferedReader readin = new BufferedReader(new InputStreamReader(input));

		Assert.assertTrue("After serialization, could not access proper mapping.", readin.readLine().equals("test"));
		input.close();
	}
}
