/*
 * Copyright (C) 2011 Andrea Paternesi, Ion Pana
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager;

import org.freelords.xmlmanager.utils.XPathHelper;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import javax.xml.bind.JAXBException;
import static org.junit.Assert.*;
import org.junit.Before;

import org.junit.Test;

/** Tests the generic XmlHelper.
 *
 * You should be able to create a test instance by unmarshalling it from an xml
 * file and marshalling it back.
 */
public class XmlManagerTest {

	private XmlManager manager;
	private File xmlFile1 = new File(this.getClass().getResource("files/simplexml.xml").getFile());
	private File xmlFile2 = new File(this.getClass().getResource("files/simplexml2.xml").getFile());
	private SimpleXml simpleXml1;
	private SimpleXml2 simpleXml2;
	private String expectedXML1;
	private String expectedXML2;

	@Before
	public void SetUp() throws Exception {
		manager = XmlTestHelper.createXmlManagerFor(SimpleXml.class, SimpleXml2.class);

		InputStream is1 = new FileInputStream(xmlFile1);
		byte buffer[] = new byte[is1.available()];
		is1.read(buffer);
		expectedXML1 = new String(buffer);

		InputStream is2 = new FileInputStream(xmlFile2);
		byte buffer2[] = new byte[is2.available()];
		is2.read(buffer2);
		expectedXML2 = new String(buffer2);
	}

	@Test
	public void XmlFileManagerCreatesTwoSimpleClassesFromXMLFiles() throws JAXBException {
		SimpleXml object1 = manager.unmarshall(xmlFile1);
		validateSimpleXml(object1);
		
		SimpleXml2 object2 = manager.unmarshall(xmlFile2);
		validateSimpleXml2(object2);
	}

	@Test
	public void XmlFileManagerCreatesTwoSimpleClassesFromXMLStreams() throws JAXBException, FileNotFoundException {
		SimpleXml object1 = manager.unmarshall(new FileInputStream(xmlFile1));
		validateSimpleXml(object1);
		
		SimpleXml2 object2 = manager.unmarshall(new FileInputStream(xmlFile2));
		validateSimpleXml2(object2);
	}
	
	private void validateSimpleXml(SimpleXml object) {
		XPathHelper helper = new XPathHelper(xmlFile1);
		assertEquals("SimpleXml unmarshall was not correct.", 
				helper.evaluateXPathAsString("//field/text()"), object.getField());
	}
	
	private void validateSimpleXml2(SimpleXml2 object) {
		XPathHelper helper = new XPathHelper(xmlFile2);
		assertEquals("SimpleXml2 unmarshall was not correct.", 
				helper.evaluateXPathAsString("//field/text()"), object.getField());
		assertEquals("SimpleXml2 unmarshall was not correct.", 
				helper.evaluateXPathAsString("//simplexml2/@attribute"), object.getAttribute());
	}
	
	@Test
	public void XmlFileManagerMarshallPlayListToByteArrayOutputStream() throws Exception {
		simpleXml1= new SimpleXml();
		simpleXml1.setField("fieldvalue");
		simpleXml2 = new SimpleXml2();
		simpleXml2.setField("fieldvalue");
		simpleXml2.setAttribute("AttributeTest");

		ByteArrayOutputStream out1 = new ByteArrayOutputStream();
		ByteArrayOutputStream out2 = new ByteArrayOutputStream();
		manager.marshal(simpleXml1, out1);
		manager.marshal(simpleXml2, out2);
		
		assertEquals("Marshalled XML was different from the expected one.", expectedXML1, out1.toString());
		assertEquals("Marshalled XML was different from the expected one.", expectedXML2, out2.toString());
	}
}
