/*
 * Copyright (C) 2011 Pana Ion, Ulf Lorenz, Andrea Paternesi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.jaxb.army;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.fail;

import java.io.File;
import java.util.UUID;

import org.freelords.army.StatCalculator;
import org.freelords.army.Unit;
import org.freelords.army.UnitStat;
import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import org.freelords.xmlmanager.utils.XPathHelper;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/** Tests serializing and deserializing of a Unit instance. */
public class UnitJaxbTest {

	private File unitFile = new File(this.getClass().getResource("files/unit.xml").getFile());
	private File outputFile = new File("unit_out.xml");

	private XmlManager xmlManager;
	private Unit unit;
	

	@Before
	public void setUp() throws Exception {
		this.xmlManager = XmlTestHelper.createXmlManagerFor(Unit.class, StatCalculator.class);
		unit = xmlManager.unmarshall(this.unitFile);
	}

	@After
	public void cleanup() {
		this.outputFile.delete();
	}

	/** Load the Unit from an xml file. */
	@Test
	public void testUnitIsReadFromAnXml() throws Exception {
		//verify that a calculator is present
		try {
			//if there is no calculator there, there should be an exception
			unit.getUnitStat(UnitStat.UPKEEP);
		} catch (Exception e) {
			fail("Calculator was not present!");
		}
		
		validateUnit(unit, new XPathHelper(this.unitFile));
	}

	/** Write the unit to an xml file. */
	@Test
	public void testUnitIsWrittenInAnXml() throws Exception {
		unit.setId(UUID.randomUUID().toString());
	  
		XmlTestHelper.writeObjectToFile(unit, this.xmlManager, this.outputFile);

		//validate
		validateUnit(unit, new XPathHelper(this.outputFile));
		validateUnitIdMarshalling(unit, new XPathHelper(this.outputFile));
	}
	
	/** Checks that the id of a unit is correctly marshalled. */
	private void validateUnitIdMarshalling(Unit unit, XPathHelper helper) {
		assertEquals("Unit id was not correctly marshalled.",
				unit.getId(), helper.evaluateXPathAsString("//id/text()"));
	}

	/** Checks that the generic data of a unit matches an xml file. */
	private void validateUnit(Unit unit, XPathHelper helper) {
		assertEquals("Unit type incorrect.",
				unit.getType(),
				helper.evaluateXPathAsString("//type/text()"));
		assertEquals("Unit movement points incorrect.",
				unit.getMovementPoints(),
				helper.evaluateXPathAsInteger("//movementPoints/text()"));
		//validate unit stats against xml
		assertEquals("Maximum move points incorrect.", 
				unit.getUnitStat(UnitStat.MAXIMUM_MOVE_POINTS), 
				helper.evaluateXPathAsInteger("//stat-calculator/stats/entry[@stat='MAXIMUM_MOVE_POINTS']/@value"));
		assertEquals("Upkeep incorrect.", 
				unit.getUnitStat(UnitStat.UPKEEP), 
				helper.evaluateXPathAsInteger("//stat-calculator/stats/entry[@stat='UPKEEP']/@value"));
	}
}
