/*
 * Copyright (C) 2011 Leonel Iturralde, Ulf Lorenz, Premkumar Bhaskal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

package org.freelords.xmlmanager.jaxb.map;

import java.io.File;
import java.util.List;

import org.freelords.army.Army;
import org.freelords.army.Unit;
import org.freelords.army.UnitStat;
import org.freelords.map.MapEntity;
import org.freelords.map.ScenarioMap;
import org.freelords.map.building.city.City;
import org.freelords.map.building.village.Village;
import org.freelords.map.tile.MapTile;
import org.freelords.reflection.ReflectionHelper;
import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import org.freelords.xmlmanager.utils.XPathHelper;

import org.junit.After;

import static junit.framework.Assert.assertEquals;
import org.junit.Test;
import org.junit.Before;

public class ScenarioMapJaxbTest {

	private XmlManager manager;
	private ScenarioMap xmlMap;
	private File inputFile = new File(this.getClass().getResource("files/map.xml").getFile());
	private File outputFile = new File("test.xml");

	@Before
	public void setUp() throws Exception {
		// all the subclasses of MapEntity must be known to JAXBContext
		manager = XmlTestHelper.createXmlManagerFor(ScenarioMap.class,
				City.class, Village.class, Army.class);
	}

	@After
	public void cleanUp() {
		outputFile.delete();
	}

	@Test
	public void testThatScenarioMapIsLoadedCorrectly() throws Exception {
		xmlMap = manager.unmarshall(inputFile);
		validateInstance(xmlMap, new XPathHelper(inputFile));
	}

	@Test
	public void testThatScenarioMapIsSavedCorrectly() throws Exception {
		xmlMap = manager.unmarshall(inputFile);
		XmlTestHelper.writeObjectToFile(xmlMap, manager, outputFile);
		validateInstance(xmlMap, new XPathHelper(outputFile));
	}

	private void validateInstance(ScenarioMap map, XPathHelper helper) {
		assertEquals("Width was not loaded correctly!", map.getWidth(),
				helper.evaluateXPathAsInteger("//width/text()"));
		assertEquals("Height was not loaded correctly!", map.getHeight(),
				helper.evaluateXPathAsInteger("//height/text()"));

		List<MapTile> maptiles = ReflectionHelper.getFieldValue(map, "tiles");

		// index to validate multiple entries of MapTile
		// also XPathHelper index is 1 based, so pre-incremented
		int maptilepos = 0;
		for (MapTile maptile : maptiles) {
			validateInstance(maptile, helper, ++maptilepos);
		}
	}

	private void validateInstance(MapTile mapTile, XPathHelper helper, int maptilepos) {
		assertEquals("MapTile was not set correctly!", mapTile.getPosition().getX(),
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos + "]/position/x/text()"));
		assertEquals("MapTile was not set correctly!", mapTile.getPosition().getY(),
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos + "]/position/y/text()"));
		assertEquals("MapTile was not set correctly!", mapTile.getName(),
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/name/text()"));
		assertEquals("MapTile was not set correctly!",mapTile.getPointsNumber(),
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos + "]/movementPoints/text()"));
		assertEquals("MapTile was not set correctly!", mapTile.getAccess().name(),
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/access/text()"));

		// checking if entity is of type Army, City or Village, then calling
		// overloaded methods appropriately.
		// multiple entry of Army, City or Village in a MapTile is not tested
		for (MapEntity entity : mapTile.getMapEntities()) {
			
			//setting the parent MapTile in the entity
			entity.setMapTile(mapTile);
			if (entity instanceof City)
				validateInstance((City) entity, helper, maptilepos);
			if (entity instanceof Army)
				validateInstance((Army) entity, helper, maptilepos);
			if (entity instanceof Village)
				validateInstance((Village) entity, helper, maptilepos);
		}
	}

	private void validateInstance(City city, XPathHelper helper, int maptilepos) {
		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/city/name/text()"), 
				city.getName());
		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/city/villages-owned/text()"), 
				city.getListOfOwnedVillages().get(0));
	}

	private void validateInstance(Village village, XPathHelper helper, int maptilepos) {
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/village/id/text()"),
				village.getId());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsBoolean("//mapTiles[position() = " + maptilepos + "]/village/siege-flag/text()"),
				village.isUnderSiege());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos + "]/village/income/text()"),
				village.getIncome());
	}

	private void validateInstance(Army army, XPathHelper helper, int maptilepos) {
		Unit unit = army.getUnits().get(0);

		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsString("//mapTiles[position() = " + maptilepos + "]/army/unit/id/text()"), 
				unit.getId());
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos + "]/army/unit/movementPoints/text()"),				
				unit.getMovementPoints());
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsString("//mapTiles[position() = "	+ maptilepos + "]/army/unit/type/text()"),
				unit.getType());
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos
						+ "]/army/unit/stat-calculator/stats/entry[@stat='MAXIMUM_MOVE_POINTS']/@value"),
				unit.getUnitStat(UnitStat.MAXIMUM_MOVE_POINTS));
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsInteger("//mapTiles[position() = " + maptilepos
						+ "]/army/unit/stat-calculator/stats/entry[@stat='UPKEEP']/@value"),
				unit.getUnitStat(UnitStat.UPKEEP));
	}
}
