/*
 * Copyright (C) 2011 Anton Kirillov, Andrea Paternesi, Ulf Lorenz, Ion Pana, Premkumar Bhaskal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.jaxb.map.tile;

import static junit.framework.Assert.assertEquals;

import java.io.File;

import junit.framework.Assert;

import org.freelords.army.Army;
import org.freelords.army.Unit;
import org.freelords.army.UnitStat;
import org.freelords.map.MapEntity;
import org.freelords.map.Position;
import org.freelords.map.building.city.City;
import org.freelords.map.tile.Access;
import org.freelords.map.tile.MapTile;
import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XPathHelper;
import org.freelords.xmlmanager.utils.XmlTestHelper;
import org.freelords.map.building.village.Village;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class MapTileJaxbTest {

	private final int x = 2;
	private final int y = 10;
	private final int movementPoints = 2;
	private final String mapTileName = "name1";
		
	private final XmlManager xmlManager = XmlTestHelper.createXmlManagerFor(
			MapTile.class, City.class, Army.class, Village.class);
	private File inputFile;
	private File outputFile = new File("mapTile_out.xml");
	private MapTile mapTile;

	@Before
	public void setUp() throws Exception {
		inputFile = new File(this.getClass().getResource("files/mapTile.xml").getFile());
		mapTile = new MapTile(mapTileName, new Position(x, y), movementPoints, Access.FORBIDDEN);
	}

	/**
	 * Test proper construction of object
	 */
	@Test
	public void testConstruction() {
		testObjectEquality(mapTile, null);
	}

	@Test
	public void testXmlUnmarshaling() throws Exception {
		testObjectEquality((MapTile) xmlManager.unmarshall(inputFile), new XPathHelper(inputFile));
	}

	@Test
	public void testMarshaling() throws Exception {
		mapTile = xmlManager.unmarshall(inputFile);
		XmlTestHelper.writeObjectToFile(mapTile, xmlManager, outputFile);
		MapTile unmarshalledObject = xmlManager.unmarshall(outputFile);
		testObjectEquality(unmarshalledObject, new XPathHelper(outputFile));
	}

	@After
	public void removeFile() {
		outputFile.delete();
	}

	private void testObjectEquality(MapTile mapTile, XPathHelper helper) {

		Assert.assertEquals(mapTile.getName(), mapTileName);
		Assert.assertEquals(mapTile.getPosition().getX(), x);
		Assert.assertEquals(mapTile.getPosition().getY(), y);
		Assert.assertEquals(mapTile.getAccess(), Access.FORBIDDEN);
		Assert.assertFalse(mapTile.canWalkOn());

		// checking if entity is of type Army, City or Village, then calling
		// overloaded methods appropriately
		// multiple entry of Army, City or Village is not tested
		for (MapEntity entity : mapTile.getMapEntities()) {

			// setting the parent MapTile in the MapEntity
			entity.setMapTile(mapTile);

			if (entity instanceof City)
				validateInstance((City) entity, helper);
			if (entity instanceof Army)
				validateInstance((Army) entity, helper);
			if (entity instanceof Village)
				validateInstance((Village) entity, helper);
		}
	}

	private void validateInstance(City city, XPathHelper helper) {

		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/x/text()"),
				city.getPosition().getX());
		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/y/text()"),
				city.getPosition().getY());

		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsString("//city/name/text()"),
				city.getName());
		assertEquals("The city is not set correctly",
				helper.evaluateXPathAsString("//city/villages-owned/text()"),
				city.getListOfOwnedVillages().get(0));
	}

	private void validateInstance(Army army, XPathHelper helper) {
		Unit unit = army.getUnits().get(0);

		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/x/text()"),
				army.getPosition().getX());
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/y/text()"),
				army.getPosition().getY());

		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsString("//army/unit/id/text()"),
				unit.getId());
		assertEquals(
				"The army is not set correctly",
				helper.evaluateXPathAsInteger("//army/unit/movementPoints/text()"),
				unit.getMovementPoints());
		assertEquals("The army is not set correctly",
				helper.evaluateXPathAsString("//army/unit/type/text()"),
				unit.getType());
		assertEquals(
				"The army is not set correctly",
				helper.evaluateXPathAsInteger("//army/unit/stat-calculator/stats/entry[@stat='MAXIMUM_MOVE_POINTS']/@value"),
				unit.getUnitStat(UnitStat.MAXIMUM_MOVE_POINTS));
		assertEquals(
				"The army is not set correctly",
				helper.evaluateXPathAsInteger("//army/unit/stat-calculator/stats/entry[@stat='UPKEEP']/@value"),
				unit.getUnitStat(UnitStat.UPKEEP));
	}

	private void validateInstance(Village village, XPathHelper helper) {

		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/x/text()"),
				village.getPosition().getX());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsInteger("//mapTile/position/y/text()"),
				village.getPosition().getY());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsString("//village/id/text()"),
				village.getId());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsBoolean("//village/siege-flag/text()"),
				village.isUnderSiege());
		assertEquals("The village is not set correctly",
				helper.evaluateXPathAsInteger("//village/income/text()"),
				village.getIncome());
	}

}
