/*
 * Copyright (C) 2011 Premkumar Bhaskal
 *               2012 Premkumar Bhaskal
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.jaxb.scenario;

import java.io.File;

import org.freelords.map.tile.Access;
import org.freelords.player.Player;
import org.freelords.scenario.Scenario;
import org.freelords.xmlmanager.XmlManager;
import org.freelords.xmlmanager.utils.XPathHelper;
import org.freelords.xmlmanager.utils.XmlTestHelper;

import static org.junit.Assert.assertEquals;
import org.junit.Before;
import org.junit.Test;

public class ScenarioJaxbTest {

	// to load the data
	private XmlManager xmlManager;
	private final int tile_x = 0;
	private final int tile_y = 0;
	private final String tile_name = "name1";
	private final int tile_movementPoints = 2;
	private final Access tile_access = Access.FORBIDDEN;

	// files for deserializing and serializing the Scenario instance
	private File scenarioFile = new File(this.getClass()
			.getResource("files/scenario.xml").getFile());
	private File outputFile = new File("scenario_out.xml");

	// needed for verifications
	private XPathHelper xPathHelper = new XPathHelper(this.scenarioFile);

	@Before
	public void setUp() throws Exception {
		this.xmlManager = XmlTestHelper.createXmlManagerFor(Scenario.class);
	}

	/**
	 * Load the Scenario from an xml file
	 * 
	 * @throws Exception
	 */
	@Test
	public void testScenarioIsReadFromAnXml() throws Exception {
		Scenario scenario = this.xmlManager.unmarshall(this.scenarioFile);
		validateScenarioUnmashall(scenario, this.xPathHelper);
	}

	/**
	 * Write the Scenario to an xml file
	 * 
	 * @throws Exception
	 */
	@Test
	public void testScenarioIsWrittenInAnXml() throws Exception {
		Scenario scenario = this.xmlManager.unmarshall(this.scenarioFile);
		XmlTestHelper.writeObjectToFile(scenario, this.xmlManager, this.outputFile);

		validateScenarioMarshall(scenario, new XPathHelper(this.outputFile));

		// clean
		removeFile(this.outputFile);
	}

	private void validateScenarioUnmashall(Scenario scenario, XPathHelper helper) {
		assertEquals("The scenario name is not set correctly",
				helper.evaluateXPathAsString("//name/text()"),
				scenario.getName());

		// testing the game map (Scenario map)
		assertEquals("The game map is not set correctly",
				helper.evaluateXPathAsInteger("//gamemap/width/text()"),
				scenario.getGamemap().getWidth());
		assertEquals("The game map is not set correctly",
				helper.evaluateXPathAsInteger("//gamemap/height/text()"),
				scenario.getGamemap().getHeight());
		assertEquals(
				"The game map is not set correctly",
				helper.evaluateXPathAsString("//gamemap/mapTiles/name/text()"),
				tile_name);
		assertEquals(
				"The game map is not set correctly",
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/position/x/text()"),
				tile_x);
		assertEquals(
				"The game map is not set correctly",
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/position/y/text()"),
				tile_y);
		assertEquals(
				"The game map is not set correctly",
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/movementPoints/text()"),
				tile_movementPoints);
		assertEquals(
				"The game map is not set correctly",
				helper.evaluateXPathAsString("//gamemap/mapTiles/access/text()"),
				tile_access.name());

		// testing the player
		Player player = scenario.getPlayers().get(0);
		assertEquals("The player is not set correctly",
				helper.evaluateXPathAsString("//title/text()"),
				player.getTitle());
		assertEquals("The player is not set correctly",
				helper.evaluateXPathAsString("//side/text()"), player.getSide()
						.toString());
	}

	private void validateScenarioMarshall(Scenario scenario, XPathHelper helper) {
		assertEquals("The scenario name is not set correctly",
				scenario.getName(),
				helper.evaluateXPathAsString("//name/text()"));

		// testing the game map (Scenario map)
		assertEquals("The game map is not set correctly", scenario.getGamemap()
				.getWidth(),
				helper.evaluateXPathAsInteger("//gamemap/width/text()"));
		assertEquals("The game map is not set correctly", scenario.getGamemap()
				.getHeight(),
				helper.evaluateXPathAsInteger("//gamemap/height/text()"));
		assertEquals(
				"The game map is not set correctly",
				tile_name,
				helper.evaluateXPathAsString("//gamemap/mapTiles/name/text()"));
		assertEquals(
				"The game map is not set correctly",
				tile_x,
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/position/x/text()"));
		assertEquals(
				"The game map is not set correctly",
				tile_y,
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/position/y/text()"));
		assertEquals(
				"The game map is not set correctly",
				tile_movementPoints,
				helper.evaluateXPathAsInteger("//gamemap/mapTiles/movementPoints/text()"));
		assertEquals(
				"The game map is not set correctly",
				tile_access.name(),
				helper.evaluateXPathAsString("//gamemap/mapTiles/access/text()"));

		// testing the player
		Player player = scenario.getPlayers().get(0);
		assertEquals("The player is not set correctly", player.getTitle(),
				helper.evaluateXPathAsString("//title/text()"));
		assertEquals("The player is not set correctly", player.getSide()
				.toString(), helper.evaluateXPathAsString("//side/text()"));
	}

	private void removeFile(File file) {
		file.delete();
	}
}
