/*
 * Copyright (C) 2011 Pana Ion, Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.utils;

import java.io.File;
import java.io.FileInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;

/** Helps with parsing DOM trees.
 *
 * The class takes an input xml file, and allows the use of xpath expressions
 * to access single entries in the xml file, and the easy conversion into
 * the various standard data types.
 *
 * See the XPathHelperTest and files/xpath.xml for examples on how to use this helper.
 */
public class XPathHelper {

	/** The xml document that we parse. */
	private Document document;
	/** A helper instance to compile expressions. */
	private XPath xPathInstance;

	/** Creates a new helper.
	 *
	 * @param file	 the xml file to parse.
	 * @throws ExceptionInInitalizerError if there were problems parsing the xml file.
	 */
	public XPathHelper(File file) {
		try {
			this.document = getDocument(file);
			this.xPathInstance = XPathFactory.newInstance().newXPath();			
		} catch (Exception e) {
			throw new ExceptionInInitializerError(e);
		}
	}

	/*
	 * Convenience methods
	 */
	public String evaluateXPathAsString(String xPath) {
		return evaluateXPath(xPath);
	}

	public int evaluateXPathAsInteger(String xPath) {
		return Integer.parseInt(evaluateXPath(xPath));
	}

	public double evaluateXPathAsDouble(String xPath) {
		return Double.parseDouble(evaluateXPath(xPath));
	}

	public boolean evaluateXPathAsBoolean(String xPath) {
		//need to do some manual manipulation because the parseBoolean function is slightly strange.
		String value = evaluateXPath(xPath);
		if (value.equals("true")) {
			return true;
		}
		if (value.equals("false")) {
			return false;
		}

		throw new IllegalArgumentException("Node was not a boolean!");
	}

	/**
	 * Evaluates an XPath expression against the document instance.
	 *
	 * @param xPathExpression   the expression to access an entry.
	 * @return the value as a String; it is further converted by the public methods.
	 * @throws RuntimeException on any errors.
	 */
	private String evaluateXPath(String xPathExpression) {
		try {
			XPathExpression expr = xPathInstance.compile(xPathExpression);
			return (String) expr.evaluate(document, XPathConstants.STRING);
		} catch (Exception e) {
			//since this will be used in the actual tests, I prefer to catch/rethrow
			//a RuntimeException to avoid try/catch blocks
			throw new RuntimeException(e);
		}
	}

	private Document getDocument(File file) throws Exception {
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
		domFactory.setNamespaceAware(true);
		DocumentBuilder builder = domFactory.newDocumentBuilder();
		return builder.parse(new FileInputStream(file));
	}
}
