/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.utils;

import java.io.File;

import org.junit.Assert;
import org.junit.Test;

/** Tests the XPathHelper class and demonstrates the use of some XPaths.
 *
 * Look at the test and compare with files/xpath.xml to see how to construct
 * XPaths.
 */
public class XPathHelperTest {

	private XPathHelper helper 
			= new XPathHelper(new File(this.getClass().getResource("files/xpath.xml").getFile()));

	@Test(expected=ExceptionInInitializerError.class)
	public void testLoadingOfInvalidFile() {
		helper = new XPathHelper(new File(helper.getClass().getResource("files/invalid.txt").getFile()));
	}

	/** Access the string in the "somestring tag", starting from the document root */
	@Test
	public void accessAStringInXml() {
		String s = helper.evaluateXPathAsString("/roottag/somestring/text()");
		Assert.assertEquals("hello", s);
	}

	/** Access and convert the integer in the "someinteger" tag; look for this tag everywhere. */
	@Test
	public void getIntegerFromSomewhereInXml() {
		int i = helper.evaluateXPathAsInteger("//someinteger/text()");
		Assert.assertEquals(5, i);
	}

	/** Access and convert the double in the "somedouble" tag somewhere below the document root. */
	@Test
	public void getDoubleFromSomewhereBelowRoot() {
		double d = helper.evaluateXPathAsDouble("/roottag//somedouble/text()");
		Assert.assertEquals(3.5, d, 0);
	}

	/** Access and convert the boolean in the "someboolean" tag somewhere below the document root. */
	@Test
	public void getBooleanFromSomewhereBelowRoot() {
		boolean b = helper.evaluateXPathAsBoolean("/roottag//someboolean/text()");
		Assert.assertEquals("Boolean not loaded properly", true, b);
	}

	/** Make sure that conversion to double fails properly. */
	public void convertInvalidDouble() {
		helper.evaluateXPathAsDouble("//somestring/text()");
	}

	/** Make sure that conversion to integer fails properly. */
	@Test(expected=java.lang.NumberFormatException.class)
	public void convertInvalidInteger() {
		helper.evaluateXPathAsInteger("//somestring/text()");
	}

	/** Make sure that conversion to boolean fails properly. */
	@Test(expected=IllegalArgumentException.class)
	public void convertInvalidBoolean() {
		helper.evaluateXPathAsBoolean("//somestring/text()");
	}

	/** If you have several children with the same name, you can use [] to access the single entities. */
	@Test
	public void accessFirstItemInASelection() {
		String s = helper.evaluateXPathAsString("/roottag/somemap/entry[1]/value/text()");
		Assert.assertEquals("1", s);
	}

	/** For the selection, you can also evaluate the value of subtags. */
	@Test
	public void accessString2InSelection() {
		String s = helper.evaluateXPathAsString("/roottag/somemap/entry[key/text() = 'string2']/value/text()");
		Assert.assertEquals("2", s);
	}
	
	/**
	 * You can access a certain element from a list using an attribute value as a filter. 
	 * It will only work if there is only 1 element returned.
	 */
	@Test
	public void accessElementBasedOnAttributeValue() {
		Assert.assertEquals("2", helper.evaluateXPathAsString("//tags/tag[@key='string1']/@value"));
	}
	
}
