/*
 * Copyright (C) 2011 Ion Pana
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.xmlmanager.utils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import junit.framework.Assert;

import org.freelords.xmlmanager.XmlManager;

public class XmlTestHelper {

	/**
	 * Helper method to write an object that is JAXB compliant to a file.
	 * 
	 * @param object the object - must be an instance of a JAXB annotated class
	 * @param xmlManager - the XmlManager to do the marshalling
	 * @param file - the output file.
	 */
	public static <T> void writeObjectToFile(T object, XmlManager xmlManager, File file) {
		if (file == null) {
			throw new IllegalArgumentException("The file to write to cannot be null!");
		}

		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		FileOutputStream fileStream = null;
		try {
			fileStream = new FileOutputStream(file);
			xmlManager.marshal(object, stream);
			stream.writeTo(fileStream);
		} catch (Exception ignored) {
			Assert.fail("Unable to marshal to file. File " + file.getAbsolutePath());
		} finally {
			if (fileStream != null) {
				try {
					fileStream.close();
				} catch (IOException e) {
					//rethrow as runtime
					throw new RuntimeException(e);
				}
			}
		}
	}

	/**
	 * Factory method for XmlManager to avoid the constructs such as 
	 * manager = new XmlManager();
	 * manager.registerClass(A.class)
	 * manager.registerClass(B.class) 
	 * ...
	 * manager.initXmlSystem()
	 * 
	 * Can be called as createXmlManagerFor(A.class, B.class).
	 * 
	 * @param clazzes
	 * @return a new initialized instance of XmlManager, with all provided classes registered
	 * @throws IllegalArgumentException if no class is provided
	 * @throws RuntimeException wrapping JAXBException should the XmlManager have troubles during initialization
	 */
	@SuppressWarnings("rawtypes")
	public static XmlManager createXmlManagerFor(Class... clazzes) {
		if (clazzes == null || clazzes.length == 0) {
			throw new IllegalArgumentException("You are supposed to provide at least one class to be registered with the XmlManager!");
		}

		XmlManager xmlManager = new XmlManager();

		for (Class clazz : clazzes) {
			//register all classes
			xmlManager.registerClass(clazz);
		}

		try {
			xmlManager.initXmlSystem();
		} catch (Exception e) {
			//just rethrow runtime exception so the client code is cleaner
			throw new RuntimeException(e);
		}
		return xmlManager;
	}
}
