/*
 * Copyright (C) 2011 Ulf Lorenz
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
package org.freelords.sound;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;

import org.freelords.resourcemanager.DataRepository;
import org.freelords.resourcemanager.InvalidFileException;
import org.freelords.resourcemanager.ResourceRepository;
import org.freelords.xmlmanager.XmlManager;

/** A simple player that can be started from the command line.
 *
 * <p>
 * It essentially takes a long list of command line arguments and works through
 * the list one by one. The syntax for the command is
 * </p>
 *
 * java org.freelords.sound.CommandLinePlayer.java &lt;commands&gt;
 *
 * where commands is a list of commands that is worked through from the left to
 * the right. Available commands are:
 *
 * <ul>
 * <li><em>&lt;file&gt;</em>: reads in and plays the sound in the given file.
 * <li><em>-d &lt;delay&gt;</em>: waits <em>delay</em> seconds before proceeding.</li>
 * <li><em>-l</em> loop the next sound</li>
 * <li><em>-f &lt;file&gt;</em> insert commands from the supplied file at this position.</li>
 * <li><em>-res &lt;file(s)&gt;</em> Set up some ResourceRepository with the
 *      file or files as root path. This is needed for setting an event map.
 *      Multiple root paths can be specified by separating them with a colon.</li>
 * <li><em>-map &lt;mapfile&gt;</em> Loads a mapping from SoundEvents to files from file.</li>
 * <li><em>-E &lt;event&gt;</em> Play a music piece referred to by the given event name.</li>
 * <li><em>-K</em> Kill the currently playing event sound.</li>
 * <li><em>-k &lt;n&gt;</em>kill the <em>n</em>-th started (non-event) sound.</li>
 * <li><em>-m</em> mute all non-event sound playing.</li>
 * <li><em>-M</em> mute all (event and non-event) sound playing</li>
 * <li><em>-um</em> resume (unmute) sound playing</li>
 * <li><em>-v &lt;volume&gt;</em> Set the sound volume to the specified value in the range 0 to 100</li>
 * <li><em>-V &lt;volume&gt;</em> Set the volume for event music to the specified value.</li>
 * </ul>
 *
 * There should be a sample file that exploits all the parameters. Just make
 * the root directory of the subversion checkout your current directory and use
 * the command line <em>-f test/org/freelords/sound/commandline.txt</em>.
 */

public class CommandLinePlayer {
	
	private AudioManager manager;
	private Iterator<String> arguments;
	private ResourceRepository resources;
	private List<String> playedSounds = new ArrayList<String>();

	private boolean loopNext = false;

	/** Runs the program. */
	public static void main(String[] args) throws Exception {
		System.out.println("Starting up");

		// Getting the correct mixer is a bit of a pain. We use the following algorithm:
		// We start with the default mixer (seems to be the first in the list). Usually, this
		// is the interface to the hardware driver. Under Linux/Alsa, Sun java seems to
		// have only one line, which is a bit too little. In this case, we try
		// to scan for the Java Audio Engine software mixer.
		Mixer mix = AudioSystem.getMixer(AudioSystem.getMixerInfo()[0]);
		if (mix.getMaxLines(new Line.Info(Clip.class)) == 1) {
			Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
			for (Mixer.Info info : mixerInfos) {
				if (info.getName() == "Java Sound Audio Engine")  {
					System.out.println("Using mixer \"" + info.getName() + "\"");
					mix = AudioSystem.getMixer(info);
					break;
				}
			}
		}
		else {
			System.out.println("Using default mixer");
		}
		AudioManager m = AudioSetup.setupAudioManager(mix);
		List<String> myargs = Arrays.asList(args);

		CommandLinePlayer player = new CommandLinePlayer(m, myargs.iterator());

		System.out.println("finished startup");

		player.decodeArguments();
	}


	/** Sets up the player with the given manager to play and commands to decode. */
	public CommandLinePlayer(AudioManager manager, Iterator<String> arguments) {
		this.manager = manager;
		this.arguments = arguments;
	}

	/** Works through the list of arguments. */
	public void decodeArguments() {
		while (arguments.hasNext()) {
			String command = arguments.next();

			if (command.equals("-d")) {
				delay();
			}
			else if (command.equals("-l")) {
				loopNext = true;
			}
			else if (command.equals("-f")) {
				readCommandsFromFile();
			}
			else if (command.equals("-res")) {
				setupResources();
			}
			else if (command.equals("-map")) {
				setupMapping();
			}
			else if (command.equals("-E")) {
				playSoundEvent();
			}
			else if (command.equals("-K")) {
				System.out.println("Killing event playing");
				manager.removeEvent();
			}
			else if (command.equals("-k")) {
				killSound();
			}
			else if (command.equals("-m")) {
				System.out.println("Muting non-event sound");
				manager.stopSound();
			}
			else if (command.equals("-M")) {
				System.out.println("Muting all sound");
				manager.stopSystem();
			}
			else if (command.equals("-um")) {
				System.out.println("Resuming sound playing");
				manager.resumeSystem();
			}
			else if (command.equals("-v")) {
				setSoundVolume();
			}
			else if (command.equals("-V")) {
				setEventVolume();
			}
			else {
				playSound(command);
			}
		}

		Runtime.getRuntime().exit(0);
	}


	/** Waits for a period of time specified in the first string of the list. */
	private void delay() {
		// catch some error
		if (!arguments.hasNext()) {
			System.out.println("No delay time given, since argument list is empty");
			return;
		}

		// convert the next argument to the delay time.
		String delay_string = arguments.next();
		int delay = 0;
		
		try {
			delay = Integer.parseInt(delay_string);
		} catch (NumberFormatException e) {
			System.out.println("Could not convert delay argument. Ignoring delay.");
		}

		// sleep for the specified number of seconds.
		System.out.println("Delaying by " + delay + "s");
		try {
			Thread.sleep(delay * 1000);
		} catch (Exception e) {
			System.out.println("Error sleeping");
		}
	}

	/** Reads commands from a file, and puts them at this position in the command queue. */
	public void readCommandsFromFile() {
		if (!arguments.hasNext()) {
			System.out.println("No file name supplied");
		}

		String commandFile = arguments.next();
		System.out.println("Reading commands from file \"" + commandFile + "\"");

		// read in the file
		String commandLine = "";
		try {
			BufferedReader input = new BufferedReader(new FileReader(commandFile));
			String line = input.readLine();
			while (line != null) {
				commandLine += line + " ";
				line = input.readLine();
			}
		} catch (FileNotFoundException e) {
			System.out.println("File not found");
		} catch (IOException e) {
			System.out.println("Error reading file");
		}

		System.out.println("Read in line: " + commandLine);

		// break the commands into pieces, assemble a new list from the
		// commands from file and from the command line, and reset the iterator
		// to this new list.
		String[] commands = commandLine.split("[ ]+");
		List<String> commandList = Arrays.asList(commands);
		while (arguments.hasNext()) {
			commandList.add(arguments.next());
		}
		arguments = commandList.iterator();
	}

	/** Sets up a resource repository used for looking up event files. */
	public void setupResources() {
		if (!arguments.hasNext()) {
			System.out.println("No directory names supplied");
		}

		String directories = arguments.next();
		System.out.println("Setting up resource with roots \"" + directories + "\"");

		resources = new DataRepository(Arrays.asList(directories.split(":")));

		try {
			resources.connectToResources();
		} catch(InvalidFileException e) {
			System.out.println("One or more resources incorrect");
			resources = null;
		}
	}

	/** Loads and sets up a mapping from SoundEvent's to music files. */
	public void setupMapping() {
		if (!arguments.hasNext()) {
			System.out.println("No mapping file provided.");
		}

		String filename = arguments.next();
		System.out.println("Loading mapping from file \"" + filename + "\"");

		XmlManager xml = new XmlManager();
		xml.registerClass(ScenarioEventSounds.class);
		try {
			xml.initXmlSystem();
		} catch (Exception e) {
			System.out.println("Error setting up xml system");
		}

		try {
			ScenarioEventSounds mapping = xml.unmarshall(new File(filename));
			mapping.setResourceRepository(resources);
			manager.setEventSounds(mapping);
		} catch(Exception e) {
			System.out.println("Error loading mapping.");
		}
	}

	public void killSound() {
		if (!arguments.hasNext()) {
			System.out.println("No sound number provided");
			return;
		}

		try {
			int sound_index = Integer.parseInt(arguments.next());
			manager.removeSound(playedSounds.remove(sound_index));
			System.out.println("Removed sound number " + sound_index);
		} catch (NumberFormatException e) {
			System.out.println("Could not convert index argument. Ignoring kill command.");
		} catch (IndexOutOfBoundsException e) {
			System.out.println("Index out of range. Ignoring kill command.");
		}
	}

	public void setSoundVolume() {
		if (!arguments.hasNext()) {
			System.out.println("No volume provided");
			return;
		}

		try {
			int volume = Integer.parseInt(arguments.next());
			manager.setSoundVolume(volume);
			System.out.println("Setting sound volume to " + volume);
		} catch(NumberFormatException e) {
			System.out.println("Cannot convert sound volume. Ignoring command.");
		}
	}

	public void setEventVolume() {
		if (!arguments.hasNext()) {
			System.out.println("No volume provided");
			return;
		}

		try {
			int volume = Integer.parseInt(arguments.next());
			manager.setEventVolume(volume);
			System.out.println("Setting event volume to " + volume);
		} catch(NumberFormatException e) {
			System.out.println("Cannot convert event volume. Ignoring command.");
		}
	}

	/** Plays an audio file associated to the given sound event. */
	public void playSoundEvent() {
		if (!arguments.hasNext()) {
			System.out.println("No event name provided.");
		}

		String eventName = arguments.next();
		System.out.println("Playing sound event \"" + eventName + "\"");

		try {
			SoundEvent event = Enum.valueOf(SoundEvent.class, eventName);
			manager.playEvent(event);
		} catch (Exception e) {
			System.out.println("Error playing event");
		}
	}

	/** Plays the given sound file. */
	private void playSound(String filename) {
		System.out.println("Playing audio file \"" + filename + "\"");
		if (loopNext) {
			System.out.println("Looping this file");
		}

		File audioFile = new File(filename);
		String ref = null;
		try {
			if (loopNext) {
				loopNext = false;
				ref = manager.playSound(new BufferedInputStream(new FileInputStream(audioFile)), true, 0, false);
			}
			else {
				ref = manager.playSound(new BufferedInputStream(new FileInputStream(audioFile)));
			}
		} catch (FileNotFoundException e) {
			System.out.println("Error reading file");
		} catch(IllegalArgumentException e) {
			System.out.println("Not a valid audio file");
		}

		if (ref == null) {
			System.out.println("Error when playing file");
		}
		else {
			playedSounds.add(ref);
		}
	}
}
